/*===========================================================================

FILE: SoundApp.c

SERVICES: Sample SOUND applet showing usage of AEE ISound interface

DESCRIPTION
  This file contains the implementation of a Sound Applet applet using the
  AEE ISound interface services.

  PUBLIC CLASSES:  
   N/A

INITIALIZATION AND SEQUENCING REQUIREMENTS:
   The following explanation applies to this sample containing one applet which serves
   as a base for app developers to create their own applets using AEE Services:
   

   In the applet source file (like this one), include AEEAppGen.h.

   Mandatory Sections in Applet Source (this file):
   -----------------------------------------------
   Following Mandatory Sections are required for each applet source file. 
   (Search for "Mandatory" to identify these sections)

   Includes:     
      Copy this section as-is from the sample applet. It contains:
      AEEAppGen.h: For AEEApplet declaration

   Type Declarations:
		A data structure must be defined to hold the app specific data. In this structure,
		the first element must be of type AEEApplet.

   Functions: (For more details, see corresponding function description in this applet)

      App_HandleEvent(): is the Event Handler to this applet.
                         Copy the function outline from the sample applet and add app specific code.

      AEEClsCreateInstance(): creates an instance of the applet and initializes it. 
                              It is called by AEEModGen when applet is being created.

   
   Important Notes:
   ---------------
   1. DO NOT use any "static data" in the applet. Always use the functions exported by
	   AEEStdlib or by IHeap services to dynamically allocate data and make it a member of 
		the applet structure. 
   2. DO NOT include and link "standard C library". Use AEE Memory Services (in AEEHeap.h) and Standard Library macros(in AEEStdLib.h).
      For example, use MALLOC() to allocate memory, WSTRCPY() to make a copy of Unicode (wide) string.
   3. BREW is Unicode(wide string) compliant ONLY (no ISOLATIN1/ANSI) except for file names which are ISOLATIN1/ANSI.
      ALWAYS USE AECHAR instead of "char". Use string services provided in AEEStdLib.h for string manipulation.
   4. It is always a good idea to DEFINE RESOURCES using BREW ResourceEditor. Make Strings, Bitmaps, Dialogs, etc. 
      as resources. ResourceEditor saves resources as .bri file, generates resource header file 
      and compiles .bri into a .bar binary file, which can be used by the applet.

   Miscellanoeus Notes:
   -------------------
   1. Make sure that the class ID used for the app is the same as that defined corresponding in .MIF file
   2. Always make sure that compiled resource (.bar) file and corresponding
      resource header (a) reside in app directory and (b) are included in the applet code.
      Define a constant APP_RES_FILE containing the name of the compiled resource file (with .bar extension).

   More than one applet:
   --------------------
   If more than one applet needs to be defined, then do the following
   (1) Follow the above description for each applet

   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR

===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services
#include "AEEStdLib.h"  // AEE StdLib Services
#include "AEEAppGen.h"	// AEEApplet structure and Applet services
#include "AEESampleAppIDs.h" // Sample App ID

// Services used by app
#include "AEEMenu.h"    // AEE Menu Services
#include "AEESound.h"   // AEE Sound Services

/*===========================================================================
                      PUBLIC DATA DECLARATIONS
===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
typedef enum
{
   SOUND_NONE,
   SOUND_PLAYTONELIST,
   SOUND_STOPTONE,
   SOUND_PLAYTONE,
   SOUND_VIBRATE,
   SOUND_STOPVIBRATE,
   SOUND_VOLUMEUP,
   SOUND_VOLUMEDOWN,
   SOUND_GETVOLUME,
   SOUND_SOUND_DEV,
   SOUND_ISHELLBEEP
} SoundOperation;

#define TONELIST_LEN    100

//Mandatory structure.
//Structure to contain app data. This is the Applet structure that will be allocated
// for the app. Define this structure if the app needs to store any data that needs
// to be remembered across different events or callbacks. The first element of this
// structure MUST be of type AEEApplet. This is extremely important. Following this
// element, store the app-specific data. Note that AEEApplet structure already exposes
// the most commonly needed interface pointers such as Shell, Display, etc. 
typedef struct _CSoundApp CSoundApp;
struct _CSoundApp
{
   AEEApplet         a;

   IMenuCtl *        m_pIMenu;
   ISound *          m_pISound;

   uint16            m_wVolume;
   uint32            m_dwTempo;
   int8              m_u8Tune;

   SoundOperation    m_eOperation;
   uint16            m_wPlayPos;
   char *            m_szOperation;
   AEESoundToneData  m_ToneArray[TONELIST_LEN];
   boolean           m_bPlayListActive;
   int               m_nPlayListIndex;

	int     m_cxWidth;
	int     m_cyHeight;
};

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean SoundApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean SoundApp_InitAppData(IApplet* po);

// App Helper Functions
// Sound callback function to be registered with ISound interface
static void SoundCBFn(
   void * pUser, 
   AEESoundCmd eCBType, 
   AEESoundStatus eSPStatus,
   uint32         dwParam
);

static void DrawRectMap( CSoundApp *pMe, int16 x, int16 y, byte* bInput );
static void DrawTextStatus( CSoundApp* pMe, AEERect rect);
static void DrawStatus( CSoundApp *pMe, AEERect rect, byte* bInput );
static void PlayBeep( CSoundApp * pMe );

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
// App specific constants
#define SOUND_APP_STRING            "Sound App"
#define SOUND_PLAYTONELIST_STRING   "Play ToneList"
#define SOUND_STOPTONE_STRING       "Stop Tone/ToneList"
#define SOUND_PLAYTONE_STRING       "Play Tone"
#define SOUND_VOLUMEUP_STRING       "Volume Up"
#define SOUND_VOLUMEDOWN_STRING     "Volume Down"
#define SOUND_VIBRATE_STRING        "Vibrate"
#define SOUND_STOPVIBRATE_STRING    "Stop Vibrate"
#define SOUND_ISHELLBEEP_STRING     "ISHELL_Beep() Demo"

#define SOUND_VOLUME_STEP           10
#define SOUND_MENU_BASE             100
#define MAX_STRLEN                  100

/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

Function:  SoundApp_InitAppData()

Description:
		This function initializes the app specific data.

Prototype:
	void SoundApp_InitAppData(IApplet* po)

Parameters:
   IApplet* po: Pointer to the applet structure. This is of the type defined
	to store applet data. (CSoundApp*, in this case).

Return Value:
	TRUE: If successful
	FALSE: IF failed

Comments:  None

Side Effects: None

==============================================================================*/
boolean SoundApp_InitAppData(IApplet* po)
{
	CSoundApp *pMe = (CSoundApp*)po;

   AEEDeviceInfo  di;

   pMe->m_pISound = NULL;
   pMe->m_wVolume = 50;
   pMe->m_dwTempo = 100;
   pMe->m_u8Tune = 0;

   pMe->m_eOperation = SOUND_NONE;
   pMe->m_wPlayPos = 0;
   pMe->m_szOperation = SOUND_STOPTONE_STRING;
   pMe->m_bPlayListActive = FALSE;
   pMe->m_nPlayListIndex = 0;

   // Get screen pixel count
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell, &di);
   pMe->m_cxWidth = di.cxScreen;
	pMe->m_cyHeight = di.cyScreen;

   return TRUE;
}

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time.

PROTOTYPE:
	int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppApplet must point to a valid IBase object. 
	If the classID	represnts an applet, then ppObj must point to a valid AEEApplet structure.Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the class/app creation was successful. 
  EFAILED: Error occurred while creating the class/app. In this case, the app/class will
  not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
	*ppObj = NULL;

   if(ClsId == AEECLSID_SOUNDAPP)
	{
		//Create the applet
      if(AEEApplet_New(sizeof(CSoundApp), ClsId, pIShell,po,(IApplet**)ppObj,(AEEHANDLER)SoundApp_HandleEvent,(PFNFREEAPPDATA)NULL))
		{
			//Initialize applet data
         if(SoundApp_InitAppData((IApplet*)*ppObj))
			{
				//Data initialized successfully
				return(AEE_SUCCESS);
			}
			else
			{
				//Release the applet. This will free the memory allocated for the applet when
				// AEEApplet_New was called.
				IAPPLET_Release((IApplet*)*ppObj);
				return EFAILED;
			}

      }//AEEApplet_New

   }// ClsId == AEECLSID_SOUNDAPP

   return(EFAILED);
}

/*===========================================================================

FUNCTION SoundApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean SoundApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AppClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
//Mandatory Function
static boolean SoundApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{
   CSoundApp * pMe = (CSoundApp *)pi;

   AEERect    menuRect;
   AECHAR szBuf[MAX_STRLEN];
   AEERect tempRect;

   SETAEERECT(&tempRect, 0, pMe->m_cyHeight*2/3, pMe->m_cxWidth, pMe->m_cyHeight/3);
   SETAEERECT(&menuRect, 0, 0, pMe->m_cxWidth, pMe->m_cyHeight - pMe->m_cyHeight/3);
	 
    switch ( eCode ) 
    {   
        case EVT_APP_START:   // Process Start event

            if (ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu))
				   return TRUE;

			   IMENUCTL_SetRect(pMe->m_pIMenu, &menuRect);  

			   IMENUCTL_Reset(pMe->m_pIMenu); 
            STR_TO_WSTR(SOUND_APP_STRING, szBuf, sizeof(szBuf));
			   IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

            STR_TO_WSTR(SOUND_PLAYTONELIST_STRING, szBuf, sizeof(szBuf));
			   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, 
               SOUND_MENU_BASE + (uint16)SOUND_PLAYTONELIST, szBuf, 0);

            STR_TO_WSTR(SOUND_STOPTONE_STRING, szBuf, sizeof(szBuf));
            IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, 
               SOUND_MENU_BASE + (uint16)SOUND_STOPTONE, szBuf, 0);

            STR_TO_WSTR(SOUND_PLAYTONE_STRING, szBuf, sizeof(szBuf));
            IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, 
               SOUND_MENU_BASE + (uint16)SOUND_PLAYTONE, szBuf, 0);

            STR_TO_WSTR(SOUND_VOLUMEUP_STRING, szBuf, sizeof(szBuf));
            IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, 
               SOUND_MENU_BASE + (uint16)SOUND_VOLUMEUP, szBuf, 0);

            STR_TO_WSTR(SOUND_VOLUMEDOWN_STRING, szBuf, sizeof(szBuf));
            IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, 
               SOUND_MENU_BASE + (uint16)SOUND_VOLUMEDOWN, szBuf, 0);

            STR_TO_WSTR(SOUND_VIBRATE_STRING, szBuf, sizeof(szBuf));
            IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, 
               SOUND_MENU_BASE + (uint16)SOUND_VIBRATE, szBuf, 0);

            STR_TO_WSTR(SOUND_STOPVIBRATE_STRING, szBuf, sizeof(szBuf));
            IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, 
               SOUND_MENU_BASE + (uint16)SOUND_STOPVIBRATE, szBuf, 0);

            STR_TO_WSTR(SOUND_ISHELLBEEP_STRING, szBuf, sizeof(szBuf));
            IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, 
               SOUND_MENU_BASE + (uint16)SOUND_ISHELLBEEP, szBuf, 0);

            IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
   		   IMENUCTL_Redraw(pMe->m_pIMenu);
            
            if ( ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_SOUND, (void **) &(pMe->m_pISound)) )
               return TRUE;
            ISOUND_RegisterNotify(pMe->m_pISound, &SoundCBFn, pMe);
				return TRUE;
            break;

        case EVT_APP_STOP:        // process STOP event
           // Cancel the callback notification
           ISOUND_RegisterNotify(pMe->m_pISound, NULL, NULL);
           ISOUND_StopTone(pMe->m_pISound);
           IMENUCTL_Release(pMe->m_pIMenu);
           ISOUND_Release(pMe->m_pISound);

           return (TRUE);
           break;

        case EVT_KEY:	    // process key event
            switch (wParam)
            {
                default:
					      return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);

            } // EVT_KEY
        case EVT_COMMAND:
               if (!pMe->m_pISound)
                  return FALSE;

		         switch( (SoundOperation) (wParam - SOUND_MENU_BASE) )
		         {
			         case SOUND_PLAYTONELIST:
                  {
                     int i;
                     AEESoundTone   eTone = AEE_TONE_0;
                     byte nRand;

                     for (i = 0; i < TONELIST_LEN; i++)
                     {
                        pMe->m_ToneArray[i].eTone = eTone;
                        pMe->m_ToneArray[i].wDuration = 150;
                        GETRAND(&nRand, 1);
                        eTone += nRand % 10;
                        if (eTone >= AEE_TONE_LAST - 10)
                           eTone = AEE_TONE_0;
                     }
/*
                     pMe->m_ToneArray[0].eTone = AEE_TONE_RING_B;
                     pMe->m_ToneArray[0].wDuration = 6000;

                     pMe->m_ToneArray[1].eTone = AEE_TONE_RING_A;
                     pMe->m_ToneArray[1].wDuration = 6000;

                     pMe->m_ToneArray[2].eTone = AEE_TONE_0;
                     pMe->m_ToneArray[2].wDuration = 4000;

                     pMe->m_ToneArray[3].eTone = AEE_TONE_1;
                     pMe->m_ToneArray[3].wDuration = 5000;
*/
                     pMe->m_eOperation = SOUND_PLAYTONELIST;
                     pMe->m_bPlayListActive = TRUE;
                     pMe->m_nPlayListIndex = 0;
                     ISOUND_PlayToneList(pMe->m_pISound, pMe->m_ToneArray, TONELIST_LEN);
							return TRUE;
                     break;
                  }

                  case SOUND_STOPTONE:
                     pMe->m_eOperation = SOUND_STOPTONE;
                     ISOUND_StopTone(pMe->m_pISound);
                     if ( pMe->m_bPlayListActive )
                     { 
                        pMe->m_bPlayListActive = FALSE;
                        pMe->m_nPlayListIndex = 0;
                     }
							return TRUE;
                     break;

			         case SOUND_PLAYTONE:
                  {
                     AEESoundToneData toneData = { AEE_TONE_0, 5000 };
                     pMe->m_eOperation = SOUND_PLAYTONE;
                     ISOUND_PlayTone(pMe->m_pISound, toneData);

							return TRUE;
                     break;
                  }

			         case SOUND_VOLUMEUP:
                  {
                     if ( pMe->m_eOperation == SOUND_NONE )
                     {
                        uint16 wVol = pMe->m_wVolume + SOUND_VOLUME_STEP;
                        wVol = wVol > AEE_MAX_VOLUME ? 0 : wVol;
                        pMe->m_eOperation = SOUND_VOLUMEUP;
                        ISOUND_SetVolume(pMe->m_pISound, wVol);
                     }
							return TRUE;
                     break;
                  }

			         case SOUND_VOLUMEDOWN:
                  {
                     if ( pMe->m_eOperation == SOUND_NONE )
                     {
                        int nVol = pMe->m_wVolume - SOUND_VOLUME_STEP;
                        nVol = nVol < 0 ? AEE_MAX_VOLUME : nVol;
                        pMe->m_eOperation = SOUND_VOLUMEDOWN;
                        ISOUND_SetVolume(pMe->m_pISound, (uint16)nVol);
                     }
			            return TRUE;
                     break;
                  }

                  case SOUND_VIBRATE:
                     pMe->m_eOperation = SOUND_NONE;
                     ISOUND_Vibrate(pMe->m_pISound, 5000);
							return TRUE;
                     break;

			         case SOUND_STOPVIBRATE:
                     pMe->m_eOperation = SOUND_NONE;
                     ISOUND_StopVibrate(pMe->m_pISound);
                     break;

                  case SOUND_ISHELLBEEP:
                     pMe->m_eOperation = SOUND_NONE;
                     pMe->m_szOperation = "BEEP_OFF";
                     DrawStatus(pMe, tempRect, NULL);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_OFF, FALSE);

                     pMe->m_szOperation = "BEEP_ALERT";
                     DrawStatus(pMe, tempRect, NULL);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_ALERT, FALSE);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_ALERT, TRUE);

                     pMe->m_szOperation = "BEEP_REMINDER";
                     DrawStatus(pMe, tempRect, NULL);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_REMINDER, FALSE);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_REMINDER, TRUE);

                     pMe->m_szOperation = "BEEP_MSG";
                     DrawStatus(pMe, tempRect, NULL);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_MSG, FALSE);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_MSG, TRUE);

                     pMe->m_szOperation = "BEEP_ERROR";
                     DrawStatus(pMe, tempRect, NULL);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_ERROR, FALSE);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_ERROR, TRUE);

                     pMe->m_szOperation = "BEEP_VIB_ALERT";
                     DrawStatus(pMe, tempRect, NULL);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_VIBRATE_ALERT, FALSE);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_VIBRATE_ALERT, TRUE);

                     pMe->m_szOperation = "BEEP_VIB_REMIND";
                     DrawStatus(pMe, tempRect, NULL);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_VIBRATE_REMIND, FALSE);
                     ISHELL_Beep(pMe->a.m_pIShell, BEEP_VIBRATE_REMIND, TRUE);

                     ISOUND_SetVolume(pMe->m_pISound, pMe->m_wVolume);

							return TRUE;
                     break;

                  default:
                     break;
               }

    }
    return FALSE;
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

static void UpdateSoundStatus( CSoundApp * pMe, AEESoundCmd eCBType, AEESoundStatus eStatus, uint32 dwParam )
{
   byte tempMap[8] = { 0x30, 0x38, 0x3c, 0x38, 0x30, 0x00, 0x00, 0x00 };
   AEERect tempRect;
   AEESoundCmdData * pData = (AEESoundCmdData *) dwParam;

   SETAEERECT(&tempRect, 0, pMe->m_cyHeight*2/3, pMe->m_cxWidth, pMe->m_cyHeight/3);

   
   switch ( eStatus )
   {
      case AEE_SOUND_SUCCESS:
         if ( pMe->m_eOperation == SOUND_VOLUMEUP || pMe->m_eOperation == SOUND_VOLUMEDOWN )
         {
            pMe->m_eOperation = SOUND_GETVOLUME;
            ISOUND_GetVolume(pMe->m_pISound);
         }
         else if ( pMe->m_eOperation == SOUND_GETVOLUME)
         {
            if ( pData )
               pMe->m_wVolume = pData->wVolume;

            pMe->m_eOperation = SOUND_PLAYTONE;
            PlayBeep(pMe);
         }

         pMe->m_szOperation = "SUCCESS";
         break;
   
      case AEE_SOUND_PLAY_DONE:
         if ( !pMe->m_bPlayListActive )
         {
            pMe->m_eOperation = SOUND_NONE;
            pMe->m_szOperation = "PLAY_DONE";
         }
         else if ( ++pMe->m_nPlayListIndex >= TONELIST_LEN )
         {
            pMe->m_eOperation = SOUND_NONE;
            pMe->m_szOperation = "PLAY_DONE";
            pMe->m_bPlayListActive = FALSE;
            pMe->m_nPlayListIndex = 0;
         }
         break;

      case AEE_SOUND_FAILURE:
         pMe->m_eOperation = SOUND_NONE;
         pMe->m_szOperation = "FAILURE";
         break;

      default:
         pMe->m_eOperation = SOUND_NONE;
         pMe->m_szOperation = "UNKNOWN";
         break;

   }

   DrawStatus(pMe, tempRect, tempMap);
}

/*===========================================================================

FUNCTION SoundCBFn

DESCRIPTION
	This is the callback function that is registered with ISound interface.
   ISound communicates with the applet via this callback only.

PROTOTYPE:
   void SoundCBFn
   (
      AEE_SOUNDCBIndType  eCBType,
      AEESoundStatus eSPStatus,
      const void *              pClientData,
      void *                    dwParam
   );

PARAMETERS:
   eCBType: Type of ISound callback.
   eSPStatus: Status contained in callback. The status depends on the 
            eCBType.
   pClientData: A reference to correlate this transaction
   dwParam: Additional info specific to callback type and status type

   Refer to BREW Reference Guide for more details.

DEPENDENCIES
  none

RETURN VALUE
  none

SIDE EFFECTS
  none
===========================================================================*/
static void SoundCBFn( void * pUser, AEESoundCmd eCBType, AEESoundStatus eStatus, uint32 dwParam )
{
   CSoundApp* pMe = (CSoundApp*)pUser;

   if ( pMe == 0 )
      return;

   UpdateSoundStatus( pMe, eCBType, eStatus, dwParam );
}


static void DrawRectMap
(
   CSoundApp* pMe,
   int16 x,
   int16 y,
   byte* bInput
)
{
#if 0
    uint8 i, j;

    for (j = 0; j < 8; j++)
    {
        if (bInput[j] == 0)
            continue;

        for (i = 0; i < 8; i++)
        {
            if (bInput[j] & (0x1 << i))
            {
                AEERect tempRect = { x + (4 * (8 - i)), y + (j * 4), 3, 3};
                IDISPLAY_DrawRect(pMe->a.m_pIDisplay, &tempRect, RGB_BLACK, RGB_BLACK, IDF_RECT_FILL);
            }
        }
    }
    IDISPLAY_Update(pMe->a.m_pIDisplay);
#endif
}

static void DrawStatus( CSoundApp *pMe, AEERect rect, byte* map )
{
   AEERect mapRect;

   SETAEERECT(&mapRect, rect.x, rect.y, rect.dx/3, rect.dy/3);
   
   IDISPLAY_EraseRgn(pMe->a.m_pIDisplay, rect.x, rect.y, rect.dx, rect.dy);

   DrawTextStatus(pMe, rect);
   DrawRectMap(pMe, mapRect.x, mapRect.y, map);
   IDISPLAY_Update(pMe->a.m_pIDisplay);
}

static void DrawTextStatus( CSoundApp* pMe, AEERect rect)
{
   AEERect textRect;

   AECHAR szBuf[MAX_STRLEN];
   AECHAR szPos[MAX_STRLEN];

   SETAEERECT(&textRect, rect.x + rect.dx/3, rect.y, rect.dx - rect.dx/3, rect.dy/3);
   STR_TO_WSTR(pMe->m_szOperation, szBuf, sizeof(szBuf));
   STR_TO_WSTR("   ", szPos, sizeof(szPos));

   IDISPLAY_EraseRgn(pMe->a.m_pIDisplay, textRect.x, textRect.y, textRect.dx, textRect.dy);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
                     szBuf, -1, 0, 0, &textRect,
                     IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE |
                     IDF_RECT_FILL | IDF_TEXT_INVERTED);
   IDISPLAY_Update(pMe->a.m_pIDisplay);

}

static void PlayBeep( CSoundApp * pMe )
{
   AEESoundToneData toneData = { AEE_TONE_SIGNAL_INDIA, 250 };
   ISOUND_PlayTone(pMe->m_pISound, toneData);
}

/*========================End of File ================================================*/
