/*===========================================================================

FILE: idialogusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of the IDialog interface. IDialog interface
  functions allow the user to create and manage user interface dialogs. The following 
  dialog-related functions are illustrated:

      -ISHELL_CreateDialog
	  -ISHELL_GetActiveDialog
	  -ISHELL_EndDialog
	  -IDIALOG_GetControl
	  -IDIALOG_SetFocus
	  -ISHELL_Prompt
	  -ISHELL_MessageBox
	  -ISHELL_MessageBoxText

  Dialogs can be created with the BREW Resource Editor or by populating a set
  of data structures in your code (the latter method is not used in these examples;
  refer to the ishell usage app for an example of this method).
  Prompts and message boxes are created by calling shell functions; they are simple
  dialogs that can be created without using the Resource Editor.

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet declaration
#include "AEEUsageAppIDs.h" // Class IDs of usage apps (including this one)
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services

// Services used by app
#include "AEEDate.h"        // AEE Date Control Services
#include "AEEMenu.h"		// Menu Services
#include "AEEStdLib.h"   // AEE stdlib services

// App resource header generated by BREW ResourceEditor.
#include "dialog_res.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// IDialog app struct. This is the main struct for this applet. This will hold 
// all the data members that need to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIDialogApp {
   AEEApplet	      a;
   IMenuCtl *  m_pIMenu;
   AEERect     m_rc;
} CIDialogApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean DialogApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IDialog_InitAppData(IApplet* pMe);
static void IDialog_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIDialogApp *pMe);
static void IDialogUsage (CIDialogApp *pMe, uint16 wParam);
static void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr);
static boolean IDialog_HandleDataEntryCmds(CIDialogApp *pMe, uint16 wParam);
static boolean IDialog_HandlePromptCmds(CIDialogApp *pMe, uint16 wParam);
static boolean IDialog_HandleStackedDialogCmds(CIDialogApp *pMe, uint16 wParam);
static boolean IDialog_InitializeDate(CIDialogApp * pMe);
static boolean DialogHandleEvent(void * pUser, AEEEvent evt, uint16 w, uint32 dw);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/

// App Resource File
#define  APP_RES_FILE      "dialog.bar"

// App specific constants
#define USAGE_BASIC_USAGE	  100
#define USAGE_CREATEDIALOG    101
#define USAGE_GETACTIVEDIALOG 102
#define USAGE_ENDDIALOG       103
#define USAGE_SETFOCUS        104
#define USAGE_GETCONTROL      105
#define USAGE_PROMPT_MENU     106
#define USAGE_PROMPT_SOFTKEY  107
#define USAGE_MESSAGEBOX      108
#define USAGE_MESSAGEBOXTEXT  109
#define	USAGE_SETEVENTHANDLER 110

// Constant used to determine size of dialog rectangle
#define SOFTKEY_MENU_HEIGHT 20

// Size of buffer used to hold text strings
#define TEXT_BUFFER_SIZE (100 * sizeof(AECHAR))

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDialog_InitAppData(). This is done as 
   follows: IDialog_InitAppData() is called to initialize the applet-specific data structure 
   instance. It is the app developer's responsibility to fill-in app data initialization code 
   for IDialog_InitAppData(). The app developer is also responsible to release memory and 
   interfaces allocated for data contained in the applet-specific data structure -- this can be
   done in IDialog_FreeAppData().

PROTOTYPE:
	boolean AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* pIModule, 
										  void** ppObj)

PARAMETERS:
	ClsId: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: [in]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  FALSE: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().  To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_DIALOG_APP){
      if(AEEApplet_New(sizeof(CIDialogApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)DialogApp_HandleEvent,(PFNFREEAPPDATA)IDialog_FreeAppData) == TRUE)
      {
         if (IDialog_InitAppData((IApplet*)*ppObj))
             return(AEE_SUCCESS);
      }
   }
   return (EFAILED);
}

/*===========================================================================

FUNCTION DialogApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function.

PROTOTYPE:
	static boolean DialogApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the IApplet structure. This structure contains information specific
	to this applet.  It was initialized during the AEEClsCreateInstance() function.

	eCode: Specifies the event sent to this applet

    wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean DialogApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIDialogApp * pMe = (CIDialogApp*)pi;

   switch (eCode) 
	{

        case EVT_APP_START:
			// When this applet is started, try to create the main menu
			// that allows the user to select a usage example to execute.
			// If we cannot create an instance of the menu class, exit and
			// indicate that we have handled the event by returning TRUE
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pIMenu) != SUCCESS)
				return TRUE;

			// Succeeded in obtaining a menu instance pointer, so
			// construct the menu and display it on the screen
			BuildMainMenu(pMe);
			return(TRUE);

        case EVT_APP_STOP:
            return(TRUE);

        case EVT_DIALOG_INIT:
			// Add dialog initialization code here
                return TRUE;
        case EVT_DIALOG_START:
			// Add dialog startup code here
                return TRUE;
        case EVT_DIALOG_END:
			// Add dialog ending code here
                return TRUE;

        case EVT_KEY: 
			switch (wParam)
			{
               case AVK_UP:
			   case AVK_DOWN:
			   case AVK_SELECT:
                  // If we receive these keys, it means that they have not been
				  // handled by an active dialog, so they are intended for our
				  // main menu control.  If there are any dialogs active, end them,
				  // clear the screen, and make sure that the main menu is active
                  if (ISHELL_GetActiveDialog(pMe->a.m_pIShell) != 0)
				  {
					  while (ISHELL_GetActiveDialog(pMe->a.m_pIShell) != 0)
						  ISHELL_EndDialog(pMe->a.m_pIShell);

                      IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &pMe->m_rc);
				  }

				  // Re-activate the main menu
				  IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);

				  // Pass the event to the main menu
			      if(IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				     return TRUE;
			      else return FALSE;

			   default:
				  // If there is a dialog active, it receives all key events.
				  // If not, see if the main menu will handle the key event
			      if(!ISHELL_GetActiveDialog(pMe->a.m_pIShell))
				  {
				      // Re-activate the main menu
				      IMENUCTL_SetActive(pMe->m_pIMenu,TRUE); 

					  if (IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				      return TRUE;
			          else return FALSE;
				  }
			}

		case EVT_COMMAND:
  		    switch(wParam)
			{
				// The following commands are generated when the user selects
				// 'Submit' or 'Change' from the soft key menu in the dialog used 
				// in examples 1, 3, 4, 5 and 6.

			    case IDI_SUBMIT:
				case IDI_CHANGE:
                    return (IDialog_HandleDataEntryCmds(pMe, wParam));

                // The following commands are generated by menu 
				// selections of the prompt dialog (example 8)
			    case IDS_YES:
			    case IDS_NO:
			    case IDS_MAYBE:
                    return (IDialog_HandlePromptCmds(pMe, wParam));

				// The following commands are generated by the stacked dialogs
				// in Example 2 (CreateDialog), in which the user can navigate
				// back and forth between the dialogs
	            case IDI_DIALOG_1_NEXT:
	            case IDI_DIALOG_2_NEXT:
				case IDI_DIALOG_1_PREV:
				case IDI_DIALOG_2_PREV:
                case IDI_DIALOG_3_PREV:
                   return (IDialog_HandleStackedDialogCmds(pMe, wParam));

				// The following commands are generated by user selections
				// from the main usage app menu.
			    case USAGE_BASIC_USAGE:
                case USAGE_CREATEDIALOG:
                case USAGE_GETACTIVEDIALOG:
                case USAGE_ENDDIALOG:
                case USAGE_SETFOCUS:
                case USAGE_GETCONTROL:
				case USAGE_PROMPT_MENU:
				case USAGE_PROMPT_SOFTKEY:
				case USAGE_MESSAGEBOX:
				case USAGE_MESSAGEBOXTEXT:
				case USAGE_SETEVENTHANDLER:

				   // De-activate menu to turn off horizontal scrolling
				   // of long menu items
				   IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

				   // Execute the usage example the user has selected
                   IDialogUsage (pMe, wParam);
			       return TRUE;

			    default:
				   return FALSE;
			}
        default:
            break;
   }
   return TRUE;
}

/*===========================================================================

FUNCTION IDialog_HandleDataEntryCmds

DESCRIPTION
	This function handles the Submit and Change commands produced by the softkey menu in
	the multi-control dialog that is used in several of the examples.  After
	the user selects Submit from the softkey menu, this function retrieves the
    the date of birth, marital status and gender from the other three controls
	in the dialog and echoes the user's selections on the screen.  The dialog is
	then ended.  When the user selects Change, focus is returned to the date control at
	the top of the screen so the user can modify the selected values in any of the
	controls.

PROTOTYPE:
    static boolean IDialog_HandleDataEntryCmds(CIDialogApp *pMe, unit16 wParam);

PARAMETERS:
	pMe: Pointer to the CIDialogApp structure. 

	wParam: Command ID passed to DialogApp_HandleEvent for this EVT_COMMAND

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the function successfully writes selection data to the screen
  FALSE: Otherwise

SIDE EFFECTS
  Clears the screen and ends the current dialog.
===========================================================================*/
static boolean IDialog_HandleDataEntryCmds(CIDialogApp *pMe, uint16 wParam)
{
   IDialog * pCurrentDialog;
   IControl * pDate, * pMarital, * pGender;
   AECHAR szDateBuffer[20];
   char szDateBuf[20], szDateStrChr[20];
   uint16 i;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return FALSE;

	switch(wParam)
	{
	    // The following command is generated when the user selects
		// 'Submit' from the soft key menu in the dialog used in example 1.
		// When this command is received, we retrieve the user's selections
		// from the dialog's other controls and echo them back to the user.
	    case IDI_SUBMIT:

		   // Get a pointer to the current dialog so we can access 
		   // its controls
		   if ((pCurrentDialog = ISHELL_GetActiveDialog(pMe->a.m_pIShell)) == 0)
			   return FALSE;

		   // Get interface pointer to the date control
		   if ((pDate = IDIALOG_GetControl(pCurrentDialog, IDC_DATE_OF_BIRTH)) == 0)
			   return FALSE;

		   // Get interface pointer to the marital-status control
		   if ((pMarital = IDIALOG_GetControl(pCurrentDialog, IDC_MARITAL_STATUS)) == 0)
			   return FALSE;

		   // Get interface pointer to the gender-selection control
		   if ((pGender = IDIALOG_GetControl(pCurrentDialog, IDC_GENDER)) == 0)
			   return FALSE;

		   IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &pMe->m_rc);	    
 
		   // Display the data the user has selected
		   DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 1, "You entered:");

		   // Retrieve date entered by user from the date control
		   IDATECTL_GetDateString((IDateCtl *) pDate, szDateBuffer, sizeof(szDateBuffer), 
			   (int *)&i, DFMT_MON_DD_YYYY);

		   // Convert date string back to a normal (non-wide) string 
		   // and add a few spaces for printing
		   WSTR_TO_STR(szDateBuffer, szDateStrChr, sizeof(szDateStrChr));
		   SPRINTF(szDateBuf, "   %s", szDateStrChr); 
           DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, szDateBuf);

		   // Print the selection
		   i = IMENUCTL_GetSel((IMenuCtl *) pMarital);
		   switch (i)
		   {
		       case IDI_SINGLE:
                  DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "   Single");
                  break;
		       case IDI_MARRIED:
                  DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "   Married");
                  break;
		       case IDI_DIVORCED:
                  DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "   Divorced");
                  break;
		       case IDI_WIDOWED:
                  DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "   Widowed");
                  break;
		       default:
                  DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "   Not Found");
			      break;
		   }

		   // Print the selection
		   i = IMENUCTL_GetSel((IMenuCtl *) pGender);
		   switch (i)
		   {
		      case IDI_SEX_MALE:
                 DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 4, "   Male");
                 break;
		      case IDI_SEX_FEMALE:
                 DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 4, "   Female");
                 break;
		      default:
                 DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 4, "   Not Found");
			     break;
		   }

           DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 5, "Press Up or");
		   DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 6, "Down key to exit");

		   // Now end the dialog (note that we had to keep the dialog
		   // around while we were accessing its controls--ending the
		   // dialog frees them)
		   ISHELL_EndDialog(pMe->a.m_pIShell);	
	       IDISPLAY_UpdateEx(pMe->a.m_pIDisplay, TRUE);
           return TRUE;

	   case IDI_CHANGE:

		  // User has selected the Change command from the softkey
		  // menu--return focus to the date control
	      if ((pCurrentDialog = ISHELL_GetActiveDialog(pMe->a.m_pIShell)) != 0)
		  {
			 IDIALOG_SetFocus(pCurrentDialog, IDC_DATE_OF_BIRTH);
			 return TRUE;
		  }
		  else return FALSE;

	   default:
		  return FALSE;
    }
}

/*===========================================================================

FUNCTION IDialog_HandlePromptCmds

DESCRIPTION
	This function handles the commands produced by the prompt dialog in
	example 8.  The user can select Yes, No or Maybe; this function creates
	a message box in response to each choice that echoes the user's selection.

PROTOTYPE:
    static boolean IDialog_HandlePromptCmds(CIDialogApp *pMe, unit16 wParam);

PARAMETERS:
	pMe: Pointer to the CIDialogApp structure. 

	wParam: Command ID passed to DialogAppHandleEvent for this EVT_COMMAND
	(for prompts, the command data value returned by a menu item selection
	is equal to the resource ID of the text string that labels that menu item).

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the function successfully displays a message box to the screen
  FALSE: Otherwise

SIDE EFFECTS
  None.
===========================================================================*/
static boolean IDialog_HandlePromptCmds(CIDialogApp *pMe, uint16 wParam)
{
   
   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   switch(wParam)
   {
      case IDS_YES:
		 ISHELL_MessageBox(pMe->a.m_pIShell, APP_RES_FILE, IDS_YES, IDS_YES_SELECTED);
		 break;

	  case IDS_NO:
		 ISHELL_MessageBox(pMe->a.m_pIShell, APP_RES_FILE, IDS_NO, IDS_NO_SELECTED);
		 break;

	  case IDS_MAYBE:
		 ISHELL_MessageBox(pMe->a.m_pIShell, APP_RES_FILE, IDS_MAYBE, IDS_MAYBE_SELECTED);
		 break;

	  default:
		 return FALSE;
   }

   // End the dialog created by the message box
   //IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
   return TRUE;
}

/*===========================================================================

FUNCTION IDialog_HandleStackedDialogCmds

DESCRIPTION
	This function handles the command produced by the menu controls in the stacked
	dialogs used in example 2 to illustrate ISHELL_CreateDialog.  These commands allow
	the user to move back and forth between the dialogs (moving back ends a dialog and
	pops the stack; moving forward creates the next dialog and pushes it onto the stack).

PROTOTYPE:
    static boolean IDialog_HandleStackedDialogCmds(CIDialogApp *pMe, unit16 wParam);

PARAMETERS:
	pMe: Pointer to the CIDialogApp structure. 

	wParam: Command ID passed to DialogAppHandleEvent for this EVT_COMMAND

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the function successfully ends or creates a dialog
  FALSE: Otherwise

SIDE EFFECTS
  Clears the screen and ends the current dialog.
===========================================================================*/
static boolean IDialog_HandleStackedDialogCmds(CIDialogApp *pMe, uint16 wParam)
{
   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   switch(wParam)
   {
      // User is at first dialog and wants to move to the second one,
	  // so create it
	  case IDI_DIALOG_1_NEXT:
		 if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_DIALOG_2, NULL) != AEE_SUCCESS)
	        return FALSE;
		 else return TRUE;

	  // User is at second dialog and wants to move to the third one,
	  // so create it
	  case IDI_DIALOG_2_NEXT:
		 if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_DIALOG_3, NULL) != AEE_SUCCESS)
            return FALSE;
		 else return TRUE;

	  // User is returning from the first dialog to the main menu--
	  // end the dialog and re-display the menu
	  case IDI_DIALOG_1_PREV:
		 ISHELL_EndDialog(pMe->a.m_pIShell);
		 IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
         IMENUCTL_Redraw(pMe->m_pIMenu);
		 return TRUE;

	  // User is returning to previous dialog--end the current one
	  case IDI_DIALOG_2_PREV:
      case IDI_DIALOG_3_PREV:
		 ISHELL_EndDialog(pMe->a.m_pIShell);
		 return TRUE;

	  default:
		 return FALSE;
   }
}
/*===========================================================================

FUNCTION IDialog_InitializeDate

DESCRIPTION
	This function initializes the date in the date control that is part of
	the data entry dialog used in several of the usage examples.  Displaying
	a non-NULL date improves the initial appearance of this control when the 
	dialog is displayed.

PROTOTYPE:
    static boolean IDialog_InitializeDate(CIDialogApp *pMe);

PARAMETERS:
	pMe: Pointer to the CIDialogApp structure. 

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the function successfully initializes the date control's date
  FALSE: Otherwise

SIDE EFFECTS
  Displays a date value in the date control of the current dialog.  If the user
  does not modify this value, it will be echoed when the user selects Submit
  from the soft key menu.
===========================================================================*/
static boolean IDialog_InitializeDate(CIDialogApp * pMe)
{
	 IDialog * pCurrentDialog;
	 IDateCtl * pDateCtl;

     // Make sure the pointers we'll be using are valid
     if (pMe == NULL || pMe->a.m_pIShell == NULL)
	    return FALSE;

     // Get a pointer to the current dialog so we can access 
     // the date control
     if ((pCurrentDialog = ISHELL_GetActiveDialog(pMe->a.m_pIShell)) == 0)
	    return FALSE;

	 // Get interface pointer to the date control
     if ((pDateCtl = (IDateCtl *) IDIALOG_GetControl(pCurrentDialog, IDC_DATE_OF_BIRTH)) == 0)
	    return FALSE;

	 // Set initial date to one that is reasonably close
     // to most people's birthdays :)
	 IDATECTL_SetDate(pDateCtl, 1970, 1, 1);
	 IDATECTL_Redraw(pDateCtl);

	 return TRUE;
}

/*===========================================================================

FUNCTION IDialog_InitAppData

DESCRIPTION
   This function initializes the fields of the applet-specific data structure.

PROTOTYPE:
	static boolean IDialog_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pMe is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialized

SIDE EFFECTS
  none
===========================================================================*/
static boolean IDialog_InitAppData(IApplet* pi)
{
   CIDialogApp * pMe = (CIDialogApp*)pi;
   AEEDeviceInfo di;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   // determine screen dimensions
   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);

   // Set the rectangle to be used for the dialogs and main menu
   pMe->m_rc.x = 0;
   pMe->m_rc.y = 0;
   pMe->m_rc.dx = di.cxScreen;
   pMe->m_rc.dy = di.cyScreen;

   pMe->m_pIMenu = NULL;

   return TRUE;
}

/*===========================================================================

FUNCTION IDialog_FreeAppData

DESCRIPTION
   Frees any data and interfaces allocated by the app during its execution.

PROTOTYPE:
	static void IDialog_FreeAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pMe is not NULL

RETURN VALUE
  None.

SIDE EFFECTS
  None.
===========================================================================*/
static void IDialog_FreeAppData(IApplet* pi)
{
   CIDialogApp * pMe = (CIDialogApp*)pi;

   if(pMe->m_pIMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: IDialogUsage

DESCRIPTION
    This function executes the usage example selected from the main menu by the
	user.  It takes as input the app data pointer and the ID of the example the
	user selected.  After clearing the screen, the selected example is executed.
	Each example is a self-contained code block that contains any local variables
	needed to execute the example.  Each code block also creates and releases any
	BREW interfaces that are required, and will exit if any errors are detected.  
	Once the example has been executed, this function prints a heading at the top 
	of the screen to indicate which example was selected, and a message at the 
	bottom of the screen instructing the user how to return to the main menu to 
	execute another example.
    
PROTOTYPE:
   static void IDialogUsage(CIDialogApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void IDialogUsage (CIDialogApp * pMe, uint16 wParam)
{
   char szResFile[] = APP_RES_FILE;   // Resource File used by this applet

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   // Erase screen first for display output purposes.
   IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &pMe->m_rc);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage: Basic Dialog Usage

			// This example loads a simple data entry dialog from a resource file.  This dialog allows the user
			// to enter their birth date, marital status and gender in a date control and two
			// list-control menus.  At the bottom of the screen, a softkey menu allows the user
			// to submit the entered data (which is then echoed back on the screen) or to change 
			// it by returning focus to the date control at the top of the screen.  See the 
			// function IDialog_HandleDataEntryCmds for the code that retrieves the entered 
			// values when the user selects Submit from the softkey menu, and for the code that 
			// changes focus (by calling IDIALOG_SetFocus) when the user selects Change.
			// NOTE: When running this example dialog, press the LEFT and RIGHT keys
			// to move between the controls in the dialog.

			// Load the dialog from this app's resource file.  Here we specify the name
			// of our resource file and the unique ID we assigned the dialog when we created
			// it in the Resource Editor.  The last parameter to ISHELL_CreateDialog is NULL,
			// since we are not creating the dialog from a data structure populated in our code.
			if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_USER_DATA_ENTRY, NULL) != AEE_SUCCESS)
				return;

			// By default, the date control in this dialog initially displays
			// a date of all zeroes.  To make the control look nicer, we will
			// start with a non-null date.
            IDialog_InitializeDate(pMe);
         }
         break;

      case USAGE_CREATEDIALOG:
         {
            // 2. Usage: ISHELL_CreateDialog

			// In this example, we will show how it is possible to create multiple dialogs
			// that are placed on the dialog stack.  Ending a dialog restores the dialog
			// (if any) immediately below it on the dialog stack.
			// We will create three dialogs.  Each dialog has a menu control that will allow the user
			// to move to the next dialog or to the previous one (for the third dialog, there
			// is no next dialog; from the first dialog, the user can return to the main menu).
			// Navigation between dialogs is handled by processing of the EVT_COMMANDs returned
			// by the menu controls (see DialogApp_HandleEvent).

			// Create the first dialog
			if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_DIALOG_1, NULL) != AEE_SUCCESS)
		       return;

			// Create the second dialog--it will be pushed onto the dialog stack, above
			// the first dialog
			if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_DIALOG_2, NULL) != AEE_SUCCESS)
		       return;

			// Create the third dialog--it will be pushed onto the dialog stack, above
			// the second dialog.  Since it is the last dialog we created, it will be the
			// one that appears on the screen initially
			if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_DIALOG_3, NULL) != AEE_SUCCESS)
               return;

			// By making selections in the menus displayed, you can move up and down in the
			// dialog stack.  When you move up, a dialog is created; when you move down, the 
			// dialog at the top of the stack is ended (see the function 
			// IDialog_HandleStackedDialogCmds for details on how this is implemented).
         }
         break; 

      case USAGE_GETACTIVEDIALOG:
         {
            // 3. Usage: ISHELL_GetActiveDialog

			// ISHELL_GetActiveDialog retrieves the IDialog interface pointer for
			// the dialog that is currently on the top of the dialog stack.  The
			// IDialog pointer is needed if you want to invoke IDIALOG_SetFocus or
			// IDIALOG_GetControl on the dialog.
			// NOTE: When running this example dialog, press the LEFT and RIGHT keys
			// to move between the controls in the dialog.
			IDialog * pCurrentDialog;

			// Display the dialog on the screen.  Note that ISHELL_CreateDialog does NOT
			// return an IDialog interface pointer to the dialog it creates (we must use
			// ISHELL_GetActiveDialog for this purpose).
			if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_USER_DATA_ENTRY, NULL) != AEE_SUCCESS)
				return;

			// Now call ISHELL_GetActiveDialog to get an interface pointer 
			// to the dialog.  We can use this pointer to call IDIALOG_SetFocus or
			// IDIALOG_GetControl (see examples for these functions).
			// See also the function IDialog_HandleDataEntryCmds (for command IDI_SUBMIT)
			// to see how ISHELL_GetActiveDialog is used to retrieve the values entered
			// in the dialog's controls.
            if ((pCurrentDialog = ISHELL_GetActiveDialog(pMe->a.m_pIShell)) == 0)
                return;

			// By default, the date control in this dialog initially displays
			// a date of all zeroes.  To make the control look nicer, we will
			// start with a non-null date.
            IDialog_InitializeDate(pMe);
         }
         break;

      case USAGE_ENDDIALOG:
         {
            // 4. Usage: ISHELL_EndDialog

			// ISHELL_EndDialog pops the active dialog from the top of the dialog
			// stack and frees any resources associated with the dialog.  To show how to use
			// this function, we first create a dialog and then end it, checking that
			// the current dialog stack is empty afterwards.

			AECHAR * szBuf, szTitleBuf[20];

			// First we create the dialog
			if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_USER_DATA_ENTRY, NULL) != AEE_SUCCESS)
				return;

			// Now end the dialog.  Since we are ending the dialog just after its creation,
			// it will not appear on the screen.  Note that ISHELL_EndDialog takes a pointer to 
			// the shell and not to a particular dialog--you can only end the dialog that is 
			// currently at the top of the dialog stack.
			if (ISHELL_EndDialog(pMe->a.m_pIShell) != SUCCESS)
				return;

			// Verify that there is no longer an active dialog, and print a message
			// if we succeeded
            if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	            return;

            STR_TO_WSTR("Dialog Ended", szTitleBuf, sizeof(szTitleBuf));
			STR_TO_WSTR("We successfully created and ended a dialog. Press Up or Down to exit.", szBuf, TEXT_BUFFER_SIZE);
            
			if (ISHELL_GetActiveDialog(pMe->a.m_pIShell) == 0)
			   ISHELL_MessageBoxText(pMe->a.m_pIShell, szTitleBuf, szBuf);

			FREE(szBuf);
         }
         break;

      case USAGE_SETFOCUS:
         {
            // 5. Usage: IDIALOG_SetFocus

			// IDIALOG_SetFocus is used to specify which control in a multi-control
			// dialog will receive input when the user presses keys.  When you create
			// a dialog with the resource editor, you can specify the control that will
			// receive focus initially, and the user can use control tabbing to move from
			// one control to another.
			// In this example, we will use IDIALOG_SetFocus to change the initial focus in
			// the dialog of Example 1 from the date control to one of the list controls.
			// NOTE: When running this example dialog, press the LEFT and RIGHT keys
			// to move between the controls in the dialog.
            // NOTE: For another example of IDIALOG_SetFocus, see the handling of the IDS_CHANGE
			// command in the function IDialog_HandleDataEntryCmds, where it is used to allow the
			// user to return to the top control in the dialog to change previously entered values.
			IDialog * pCurrentDialog;
			int16 wPrevFocusCtl;

			// First create the dialog
			if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_USER_DATA_ENTRY, NULL) != AEE_SUCCESS)
				return;

			// Get an interface pointer to the dialog, which we'll need to change 
			// the dialog's focus
            if ((pCurrentDialog = ISHELL_GetActiveDialog(pMe->a.m_pIShell)) == 0)
                return;

			// Now change focus to the gender (Male/Female) list control, which is
			// third from the top (the control ID, IDC_GENDER, was assigned to this
			// control when we created it in the Resource Editor).  Note that IDIALOG_SetFocus
			// returns the ID of the control that previously had the focus.
            wPrevFocusCtl = IDIALOG_SetFocus(pCurrentDialog, IDC_GENDER);

			// By default, the date control in this dialog initially displays
			// a date of all zeroes.  To make the control look nicer, we will
			// start with a non-null date.
            IDialog_InitializeDate(pMe);
         }
         break;

      case USAGE_GETCONTROL:
         {
            // 6. Usage: IDIALOG_GetControl

			// IDIALOG_GetControl allows you to get interface pointers to the controls that
			// are part of the dialog.  This allows you to modify the appearance of the
			// controls and retrieve entered values by invoking control interface functions.
			// To illustrate how to use IDIALOG_GetControl, we will use it to adjust the
			// size and positions of the controls in the dialog used in Example 1.  Note
			// that in the Resource Editor, you have to enter constant values for the control
			// origin, height and width.  Here we will do a better job by taking into account
			// the available screen real estate and the actual sizes of the controls.
			// NOTE: When running this example dialog, press the LEFT and RIGHT keys
			// to move between the controls in the dialog.
			IDialog * pCurrentDialog;			
			IControl * pDate;
			IControl * pMarital;
			IControl * pGender;
			IControl * pSubmit;
			AEERect rc_date, rc_marital, rc_gender, rc_submit; // screen and control rectangles
			uint16 cLineHeight, cInterCtlSpace;
			AECHAR szBuf[20];

			// First create the dialog
			if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_USER_DATA_ENTRY, NULL) != AEE_SUCCESS)
				return;
			
			// Get a pointer to the dialog, and use it to get pointers to each of the
			// dialog's controls.  Note that IDIALOG_GetControl returns 0 if it cannot
			// find the control with the specified ID in the dialog.
			if ((pCurrentDialog = ISHELL_GetActiveDialog(pMe->a.m_pIShell)) != 0)
			{
		        // Get an interface pointer to the Date control
				if ((pDate = IDIALOG_GetControl(pCurrentDialog, IDC_DATE_OF_BIRTH)) == 0)
					return;

		        // Get an interface pointer to the Marital Status control
				if ((pMarital = IDIALOG_GetControl(pCurrentDialog, IDC_MARITAL_STATUS)) == 0)
					return;

		        // Get an interface pointer to the Gender control
				if ((pGender = IDIALOG_GetControl(pCurrentDialog, IDC_GENDER)) == 0)
					return;

		        // Get an interface pointer to the Submit/Change control
				if ((pSubmit = IDIALOG_GetControl(pCurrentDialog, IDC_SUBMIT)) == 0)
					return;
			}
            else return;

			// Clear the screen (we'll redraw each control after calculating its
			// new rectangle)
		    IDISPLAY_EraseRect(pMe->a.m_pIDisplay, &pMe->m_rc);

			// Determine how much vertical space to leave between controls

			// Height of a line of normal text (add 2 pixels for spacing)
            cLineHeight = 2 + IDISPLAY_GetFontMetrics(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, NULL, NULL);

			// A date control uses two text lines, and each list control uses one line.
			// Divide the available unused space among the three inter-control spaces
			// between the four controls
			cInterCtlSpace = (pMe->m_rc.dy - 4 * cLineHeight - SOFTKEY_MENU_HEIGHT) / 3;

			// Compute the rectangle for the date control
			rc_date.x  = pMe->m_rc.x;
			rc_date.y  = pMe->m_rc.y;
			rc_date.dx = pMe->m_rc.dx;
			rc_date.dy = 2 * cLineHeight;

			// Set the rectangle of the date control to its new value
			ICONTROL_SetRect(pDate, &rc_date);

			// Set a value for the initial date displayed by the date control
			// (prevents empty control contents)
			IDATECTL_SetDate((IDateCtl *)pDate, 1970, 1, 1);
			ICONTROL_Redraw(pDate);

			// Compute the rectangle for the marital status control
			rc_marital.x  = pMe->m_rc.x;
			rc_marital.y  = rc_date.y + rc_date.dy + cInterCtlSpace;

			// To compute control width, use the width of the longest string
			// among the list control's items
			STR_TO_WSTR("Widowed", szBuf, sizeof(szBuf));
			rc_marital.dx = 2 + IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, pMe->m_rc.dx, NULL);
			rc_marital.dy = cLineHeight;

			// Set the rectangle of the marital-status control to its new value
			ICONTROL_SetRect(pMarital, &rc_marital);
			ICONTROL_Redraw(pMarital);

			// Compute the rectangle for the gender control
			rc_gender.x  = pMe->m_rc.x;
			rc_gender.y  = rc_marital.y + rc_marital.dy + cInterCtlSpace;

			// To compute control width, use the width of the longest string
			// among the list control's items
			STR_TO_WSTR("Female", szBuf, sizeof(szBuf));
			rc_gender.dx = 2 + IDISPLAY_MeasureTextEx(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1, pMe->m_rc.dx, NULL);
			rc_gender.dy = cLineHeight;

			// Set the rectangle of the gender control to its new value and redraw it
			ICONTROL_SetRect(pGender, &rc_gender);
			ICONTROL_Redraw(pGender);

			// Compute the rectangle for the submit control--since it is a
			// softkey menu, we place it flush with bottom of screen
			rc_submit.x  = pMe->m_rc.x;
			rc_submit.y  = pMe->m_rc.y + pMe->m_rc.dy - SOFTKEY_MENU_HEIGHT;
			rc_submit.dx = pMe->m_rc.dx;
			rc_submit.dy = SOFTKEY_MENU_HEIGHT;

			// Set the rectangle of the submit control to its new value and redraw it
			ICONTROL_SetRect(pSubmit, &rc_submit);
			ICONTROL_Redraw(pSubmit);
         }
         break; 
		 
	  case USAGE_PROMPT_MENU:
         {
            // 7. Usage: ISHELL_Prompt, standard menu with title

			// This example illustrates the use of the ISHELL_Prompt function.  To
			// use this function, you populate the fields of an AEEPromptInfo structure
			// and then pass a pointer to the structure to ISHELL_Prompt.  In this example,
			// a NULL value is specified for the prompt text, which causes the prompt selections
			// to be displayed as a standard menu (if text is specified, a softkey menu is used,
			// as shown in the example below).

			AEEPromptInfo pi;
			uint16 wButtonIDs[] = {IDS_YES, IDS_NO, IDS_MAYBE};
        
			// Populate fields of the prompt info data structure

			// Name of our resource file
			pi.pszRes = szResFile;

			// You can specify the text and title strings with character-string pointers.
			// Since we will read the text and title from the resource file, we set these
			// pointers to NULL.
            pi.pTitle = NULL;
            pi.pText = NULL;

			// Resource ID of the title for this example
            pi.wTitleID = IDS_PROMPT_TITLE;

			// We are not using the text message in this example, so set
			// this value to zero.
            pi.wTextID = 0;

			// Specify the default button (this will be the selection if the
			// prompt times out before the user has made a selection)
            pi.wDefBtn = IDS_NO;

			// Specify the button IDs, which will be presented as menu selections
			// from which the user can make a choice.
            pi.pBtnIDs = wButtonIDs;

			// The text and title are presented using a static text control (instance
			// of the IStatic interface). You can specify the control's properties here
			// (we choose to center the static text control's title).
            pi.dwProps = ST_CENTERTITLE;

			// Specify the font to be used to display the title
            pi.fntTitle = AEE_FONT_BOLD;

			// Since we're not using any text, font doesn't matter
            pi.fntText = 0;

			// Prompts have an optional timeout value--if the amount of time specified
			// passes before the user has made a selection, the selection identified
			// in the default button ID is made automatically and the prompt dialog is
			// dismissed.  Here, we set the timeout value to 0, which prevents automatic
			// selection (the prompt dialog remains on the screen until the user makes
			// a selection or exits).  See next example for a non-zero timeout value.
            pi.dwTimeout = 0;  

			// Now that the AEEPromptInfo structure has been populated, create
			// and display the prompt
			if (ISHELL_Prompt(pMe->a.m_pIShell, &pi) == FALSE)
				return;

			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay, TRUE);
         }
         break;

	  case USAGE_PROMPT_SOFTKEY:
         {
            // 8. Usage: ISHELL_Prompt, softkey menu with static text control above

			// This example illustrates the use of the ISHELL_Prompt function.  To
			// use this function, you populate the fields of an AEEPromptInfo structure
			// and then pass a pointer to the structure to ISHELL_Prompt.  In this example,
			// a non-NULL value is specified for the prompt text, which causes the prompt selections
			// to be displayed as a softkey menu below a static text control containing the
			// title and text (if text is not specified, a standard menu is used,
			// as shown in the example above).

			AEEPromptInfo pi;
			uint16 wButtonIDs[] = {IDS_YES, IDS_NO, IDS_MAYBE};
        
			// Populate fields of the prompt info data structure

			// Name of our resource file
			pi.pszRes = szResFile;

			// You can specify the text and title strings with character-string pointers.
			// Since we will read the text and title from the resource file, we set these
			// pointers to NULL.
            pi.pTitle = NULL;
            pi.pText = NULL;

			// Resource ID of the title for this example
            pi.wTitleID = IDS_PROMPT_TITLE;

			// Resource ID of the text for this example
            pi.wTextID = IDS_PROMPT_TEXT;

			// Specify the default button (this will be the selection if the
			// prompt times out before the user has made a selection)
            pi.wDefBtn = IDS_NO;

			// Specify the button IDs, which will be presented as menu selections
			// from which the user can make a choice.
            pi.pBtnIDs = wButtonIDs;

			// The text and title are presented using a static text control (instance
			// of the IStatic interface). You can specify the control's properties here
			// (we choose to center the static text control's title and vertically center
			// the prompt text).
            pi.dwProps = ST_MIDDLETEXT | ST_CENTERTITLE;

			// Specify the font to be used to display the title
            pi.fntTitle = AEE_FONT_BOLD;

			// Specify the font to be used to display the text
            pi.fntText = AEE_FONT_NORMAL;

			// Prompts have an optional timeout value--if the amount of time specified
			// passes before the user has made a selection, the selection identified
			// in the default button ID is made automatically and the prompt dialog is
			// dismissed.  Here, we set the timeout value to 10000 (10 seconds). 
			// See above example for a zero timeout value.
            pi.dwTimeout = 10000;  

			// Now that the AEEPromptInfo structure has been populated, create
			// and display the prompt
			if (ISHELL_Prompt(pMe->a.m_pIShell, &pi) == FALSE)
				return;

			IDISPLAY_UpdateEx(pMe->a.m_pIDisplay, TRUE);
         }
         break;

	  case USAGE_MESSAGEBOX:
         {
            // 9. Usage: ISHELL_MessageBox

			// A message box is a screen containing a title and a text message that
			// is dismissed when the user presses any key.  The ISHELL_MessageBox function
			// creates a dialog that displays the title and text with a single function call
			// (you do not need to create or release any other interfaces).  The function reads
			// the title and text strings from a resource file (the function ISHELL_MessageBoxText
			// lets you specify the text strings in your code (see below)).
            if (ISHELL_MessageBox(pMe->a.m_pIShell, APP_RES_FILE, 
				IDS_MESSAGEBOX_TITLE, IDS_MESSAGEBOX_TEXT) == FALSE)
				return;
			
			// End the dialog created by the message box so that user can
			// return to main menu with a single key press
			ISHELL_EndDialog(pMe->a.m_pIShell);	
         }
         break;

	  case USAGE_MESSAGEBOXTEXT:
         {            
			// 10. Usage: ISHELL_MessageBoxText

			// A message box is a screen containing a title and a text message that
			// is dismissed when the user presses any key.  The ISHELL_MessageBoxText function
			// creates a dialog that displays the title and text with a single function call
			// (you do not need to create or release any other interfaces).  Pointers to the
			// title and text strings are passed to the function (the function ISHELL_MessageBox
			// lets you read the title and text strings from a resource file (see above)).

			AECHAR szTitleBuf[20], * szTextBuf;
			STR_TO_WSTR("MessageBoxText", szTitleBuf, sizeof(szTitleBuf));

			if ((szTextBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	            return;

			STR_TO_WSTR("This is a message box created with MessageBoxText.  Press Up or Down key to exit.",
				szTextBuf, TEXT_BUFFER_SIZE);

            if (ISHELL_MessageBoxText(pMe->a.m_pIShell, szTitleBuf, szTextBuf) == FALSE)
			{
				FREE(szTextBuf);
				return;
			}

			// End the dialog created by the message box so that user can
			// return to main menu with a single key press
			ISHELL_EndDialog(pMe->a.m_pIShell);	

            FREE(szTextBuf);
         }
         break;

	  case USAGE_SETEVENTHANDLER:
         {            
			// 11. Usage: IDIALOG_SetEventHandler
			IDialog * pCurrentDialog;

			// Display the dialog on the screen.  Note that ISHELL_CreateDialog does NOT
			// return an IDialog interface pointer to the dialog it creates (we must use
			// ISHELL_GetActiveDialog for this purpose).
			if (ISHELL_CreateDialog(pMe->a.m_pIShell, APP_RES_FILE, IDD_USER_DATA_ENTRY, NULL) != AEE_SUCCESS)
				return;

			// Now call ISHELL_GetActiveDialog to get an interface pointer 
			// to the dialog.  We can use this pointer to call IDIALOG_SetFocus or
			// IDIALOG_GetControl (see examples for these functions).
			// See also the function IDialog_HandleDataEntryCmds (for command IDI_SUBMIT)
			// to see how ISHELL_GetActiveDialog is used to retrieve the values entered
			// in the dialog's controls.
            if ((pCurrentDialog = ISHELL_GetActiveDialog(pMe->a.m_pIShell)) == 0)
                return;

			// By default, the date control in this dialog initially displays
			// a date of all zeroes.  To make the control look nicer, we will
			// start with a non-null date.
            IDialog_InitializeDate(pMe);

			// By default dialog related events are sent to the app that owns the dialog.
			// If an event handler is specified using IDIALOG_SetEventHandler, events are
			// to that event handler.
			// These events are:
			// 1. EVT_COPYRIGHT_END   (for ISHELL_Copyright dialogs)
			// 2. EVT_DIALOG_END
			// 3. EVT_COMMAND with ID of the button that caused dialog to close 
			//    (in the case of ISHELL_Prompt where u have an ID associated with the CLR button 
			//    i.e. with wDefButton).


			IDIALOG_SetEventHandler(pCurrentDialog, DialogHandleEvent, pMe);
         }
         break;

      default:
         return;
   }
            
   return;
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function constructs the menu control that allows the user to
	select a usage example to execute.  The command ID for each menu item
	is passed to the usage example function above when the user selects the
	menu item for that example.
    
PROTOTYPE:
   static void BuildMainMenu(CIDialogApp * pMe)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIDialogApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
    AECHAR * szBuf;

	// Make sure the pointers we'll be using are valid
    if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->m_pIMenu == NULL)
	   return;

    if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return;

	// Set Title
    STR_TO_WSTR("IDialog Examples", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	//Add individual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add CreateDialog to menu
   STR_TO_WSTR("2. CreateDialog", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CREATEDIALOG, szBuf, 0);
			
   // Add GetActiveDialog to menu
   STR_TO_WSTR("3. GetActiveDialog", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETACTIVEDIALOG, szBuf, 0);

   // Add EndDialog to menu
   STR_TO_WSTR("4. EndDialog", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_ENDDIALOG, szBuf, 0);

   // Add SetFocus to menu
   STR_TO_WSTR("5. SetFocus", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETFOCUS, szBuf, 0);

   // Add GetControl to menu
   STR_TO_WSTR("6. GetControl", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETCONTROL, szBuf, 0);	

   // Add Menu Prompt to menu
   STR_TO_WSTR("7. Menu Prompt", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PROMPT_MENU, szBuf, 0);	

   // Add Softkey Prompt to menu
   STR_TO_WSTR("8. Softkey Prompt", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_PROMPT_SOFTKEY, szBuf, 0);	

   // Add MessageBox to menu
   STR_TO_WSTR("9. MessageBox", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_MESSAGEBOX, szBuf, 0);	

   // Add MessageBoxText to menu
   STR_TO_WSTR("10. MessageBoxText", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_MESSAGEBOXTEXT, szBuf, 0);	

   // Add MessageBoxText to menu
   STR_TO_WSTR("11. SetEventHandler", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_SETEVENTHANDLER, szBuf, 0);
	
   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);

	FREE(szBuf);
}
/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr)

PARAMETERS:
   pIShell:   [in]: Contains a pointer to the IShell interface.
   pIDisplay: [in]: Contains a pointer to the IDisplay interface.
   nline:     [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   // Make sure the pointers we'll be using are valid
   if (pIShell == NULL || pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE)) == NULL)
	   return;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, TEXT_BUFFER_SIZE);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
	  {
		 FREE(szBuf);
         return;
	  }

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

   FREE(szBuf);
   return;   

} // End of DisplayOutput

boolean DialogHandleEvent(void * pUser, AEEEvent evt, uint16 w, uint32 dw)
{
	CIDialogApp* pMe = (CIDialogApp*)pUser;
	switch (evt)
	{
		case EVT_DIALOG_END:
			DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 8, "DialogHandleEvent Invoked for EVT_DIALOG_END");
			break;

		default:
			break;

	}
	return TRUE;
}
