/*===========================================================================

FILE: iheapusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of the IHeap interface. IHeap interface
  functions allocate and free blocks of memory and return information about the
  amount of memory that is available and in use in the device.  The following 
  functions are illustrated in this usage example:

      -IHEAP_CheckAvail
	  -IHEAP_Free
	  -IHEAP_GetMemStats
	  -IHEAP_Malloc
	  -IHEAP_MallocRec
	  -IHEAP_Realloc
	  -IHEAP_StrDup

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
// Mandatory includes
#include "AEEAppGen.h"	// AEEApplet declaration
#include "AEEUsageAppIDs.h" // Class IDs of usage apps (including this one)
#include "AEE.h"			// Standard AEE Declarations
#include "AEEShell.h"	// AEE Shell Services
#include "AEEDisp.h"		// AEE Display Services

// Services used by this app
#include "AEEHeap.h"		//	AEEHeap Services
#include "AEEMenu.h"		// Menu Services
#include "AEEStdLib.h"   // AEE StdLib services

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// IHeap app struct. This is the main struct for this applet. This will hold 
// all the data members that need to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CIHeapApp {
   AEEApplet	      a;
   IMenuCtl *  m_pIMenu;
   AEERect     m_rc;
} CIHeapApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// Mandatory function:
// App Handle Event function
static boolean HeapApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean IHeap_InitAppData(IApplet* pMe);
static void IHeap_FreeAppData(IApplet* pMe);

static void BuildMainMenu(CIHeapApp *pMe);
static void IHeapUsage (CIHeapApp *pMe, uint16 wParam);
static void DisplayEvent (CIHeapApp *pMe, uint16 wParam);
static void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr);
/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/

// App specific constants
#define USAGE_BASIC_USAGE  100
#define USAGE_CHECKAVAIL   101
#define USAGE_FREE         102
#define USAGE_GETMEMSTATS  103
#define USAGE_MALLOC       104
#define USAGE_MALLOCREC    105
#define USAGE_REALLOC      106
#define USAGE_STRDUP       107
#define USAGE_ERASE_SCREEN 108   // Special case to erase screen

// Size of buffer used to hold text strings
#define TEXT_BUFFER_SIZE (100 * sizeof(AECHAR))

/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/


/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/


/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IHeap_InitAppData(). This is done as follows:
   IHeap_InitAppData() is called to initialize AppletData instance. It is app developers 
   responsibility to fill-in app data initialization code of IHeap_InitAppData(). App developer
   is also responsible to release memory allocated for data contained in AppletData -- this can be
   done in IHeap_FreeAppData().

PROTOTYPE:
	boolean AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* pIModule, 
										  void** ppObj)

PARAMETERS:
	ClsId: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid AEEApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  FALSE: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId, IShell* pIShell, IModule* po, void** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_HEAP_APP){
      if(AEEApplet_New(sizeof(CIHeapApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)HeapApp_HandleEvent,(PFNFREEAPPDATA)IHeap_FreeAppData) == TRUE)
      {
         if(IHeap_InitAppData((IApplet*)*ppObj))
             return(AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION HeapApp_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. 

PROTOTYPE:
	static boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet.

	ecode: Specifies the Event sent to this applet

    wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/

static boolean HeapApp_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CIHeapApp * pMe = (CIHeapApp*)pi;

   switch (eCode) 
	{

        case EVT_APP_START:
			// When this applet is started, try to create the main menu
			// that allows the user to select a usage example to execute.
			// If we cannot create an instance of the menu class, exit and
			// indicate that we have handled the event by returning TRUE
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, 
				  (void **)&pMe->m_pIMenu) != SUCCESS)
				return TRUE;

			// Succeeded in obtaining a menu instance pointer, so
			// construct the menu and display it on the screen
			BuildMainMenu(pMe);
			return(TRUE);

        case EVT_APP_STOP:
            return(TRUE);

        case EVT_KEY:
			  if (pMe->m_pIMenu)
			  {
					if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && ((wParam == AVK_UP) || (wParam == AVK_DOWN)))
					{
						IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
						IMENUCTL_Redraw(pMe->m_pIMenu);
					}
			      return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			  }
			  else 
				  return FALSE;

		case EVT_COMMAND:
  		    switch(wParam)
			{
				// The following commands are generated by user selections
				// from the main usage app menu.
			    case USAGE_BASIC_USAGE:
                case USAGE_CHECKAVAIL:
                case USAGE_FREE:
                case USAGE_GETMEMSTATS:
                case USAGE_MALLOC:
                case USAGE_MALLOCREC:
				case USAGE_REALLOC:
                case USAGE_STRDUP:

				   // De-activate main menu to turn off horizontal scrolling
				   // of long menu items
				   IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

				   // Execute the usage example the user has selected
                   IHeapUsage (pMe, wParam);
			       return TRUE;
			    default:
				    return FALSE;
			}
        default:
            break;
   }
   return TRUE;
}


/*===========================================================================

FUNCTION IHeap_InitAppData

DESCRIPTION
   This function initializes app-specific data.

PROTOTYPE:
	static boolean IHeap_InitAppData(IApplet* pi);

PARAMETERS:
	pi [in]: Pointer to the IApplet structure. This structure contains information specific
	to this applet.

DEPENDENCIES
   None.

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialized

SIDE EFFECTS
  none
===========================================================================*/
static boolean IHeap_InitAppData(IApplet* pi)
{
   CIHeapApp * pMe = (CIHeapApp*)pi;

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL)
	   return FALSE;

   pMe->m_pIMenu = NULL;

   return TRUE;
}

/*===========================================================================

FUNCTION IHeap_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.

PROTOTYPE:
   static boolean IHeap_FreeAppData(IApplet* pi);

PARAMETERS:
   pi [in]: Pointer to the IApplet structure. This structure contains information specific
   to this applet.

DEPENDENCIES
   None.

RETURN VALUE
   None.

SIDE EFFECTS
   None.
===========================================================================*/
static void IHeap_FreeAppData(IApplet* pi)
{
   CIHeapApp * pMe = (CIHeapApp*)pi;

   if(pMe->m_pIMenu != NULL)
   {
	  IMENUCTL_Release(pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

FUNCTION: IHeapUsage

DESCRIPTION
    This function executes the usage example selected from the main menu by the
	user.  It takes as input the app data pointer and the ID of the example the
	user selected.  After clearing the screen, the selected example is executed.
	Each example is a self-contained code block that contains any local variables
	needed to execute the example.  Each code block also creates and releases any
	BREW interfaces that are required, and will exit if any errors are detected.  
	Once the example has been executed, this function prints a heading at the top 
	of the screen to indicate which example was selected, 
	and a message at the bottom of the screen instructing the user how to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void IHeapUsage(CIHeapApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void IHeapUsage (CIHeapApp * pMe, uint16 wParam)
{
   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   // Erase screen first for display output purposes.
   DisplayEvent (pMe, USAGE_ERASE_SCREEN);
	IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
	IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
	  case USAGE_FREE:
      case USAGE_MALLOC:
      case USAGE_MALLOCREC:
         {
			// 1. Usage: Basic Heap Usage
			// 2. Usage: IHEAP_Free
            // 3. Usage: IHEAP_Malloc
            // 4. Usage: IHEAP_MallocRec

			// These functions are used to allocate and free blocks of memory of a given size.
			// IHEAP_MallocRec is used to allocate enough memory for a single instance
			// of a specified data type.  To illustrate these functions, we first use
			// IHEAP_Malloc to allocate memory for an AEEDeviceInfo structure, which is then
			// used to retrieve device info.  We then repeat same the task using 
			// IHEAP_MallocRec.  In both cases, we use IHEAP_Free to free the allocated
			// memory blocks before exiting.

			IHeap * pIHeap;
	        char szBuf[30];
			AEEDeviceInfo *di;

			// Create an instance of the IHeap interface
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_HEAP, 
				  (void **)&pIHeap) != SUCCESS)
				return;

			// Allocate memory for an AEEDeviceInfo structure
			if ((di = (AEEDeviceInfo *) IHEAP_Malloc(pIHeap, sizeof(AEEDeviceInfo))) == NULL)
				return;

			// Now use the allocated memory to retrieve device info data
			ISHELL_GetDeviceInfo(pMe->a.m_pIShell, di);

			// Echo back some of the data from the device-info structure
		    SPRINTF(szBuf, "cx: %d cy: %d", di->cxScreen, di->cyScreen); 
            DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, szBuf);

			// Free the memory we just allocated
			IHEAP_Free(pIHeap, di);

			// We now repeat the above example using IHEAP_MallocRec instead of 
			// IHEAP_Malloc.

			// Allocate memory for an AEEDeviceInfo structure
			if ((di = IHEAP_MallocRec (pIHeap, AEEDeviceInfo)) == NULL)
				return;

			// Now use the allocated memory to retrieve device info data
			ISHELL_GetDeviceInfo(pMe->a.m_pIShell, di);

			// Echo back some of the data from the device-info structure
		    SPRINTF(szBuf, "cx: %d cy: %d", di->cxScreen, di->cyScreen); 
            DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, szBuf);

			// Free the memory we just allocated
			IHEAP_Free(pIHeap, di);

			// Free the instance of the IHeap interface
			IHEAP_Release(pIHeap);
         }
         break;

     case USAGE_CHECKAVAIL:
         {
            // 5. Usage: IHEAP_CheckAvail

			// This function checks if a given amount of memory is available
			// to be allocated, without actually allocating the memory.
			// As an example of how to use this function, we first try it with
			// a small amount of memory and then with a very large amount, thereby
			// illustrating both successful and unsuccessful return values of the function.
			IHeap * pIHeap;

			// Create an instance of the IHeap interface
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_HEAP, 
				  (void **)&pIHeap) != SUCCESS)
				return;

			// Check if a small block of memory can be allocated--this should succeed
			if (IHEAP_CheckAvail(pIHeap, 50) == TRUE)
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, "Can allocate 50 bytes");
			else DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, "Can't allocate 50 bytes");

			// Now check if a huge block (1 Gb) of memory can be allocated--this should fail
			if (IHEAP_CheckAvail(pIHeap, 1000000000) == TRUE)
                DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "Can allocate 1 Gb");
			else DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, "Can't allocate 1 Gb");

			// Free the instance of the IHeap interface
			IHEAP_Release(pIHeap);
         }
         break; 

      case USAGE_GETMEMSTATS:
         {
            // 6. Usage: IHEAP_GetMemStats

			// This function returns the amount of memory that is currently in
			// use in the device.  We illustrate the function by using it to obtain
			// the memory in use and echoing the returned value to the screen.

			IHeap * pIHeap;
            uint32 dwMemInUse;
			char szBuf[50];

			// Create an instance of the IHeap interface
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_HEAP, 
				  (void **)&pIHeap) != SUCCESS)
				return;

			// Determine the total amount of memory currently in use
            dwMemInUse = IHEAP_GetMemStats(pIHeap);

			// Echo back the amount of memory in use
		    SPRINTF(szBuf, "Memory in use: %d bytes", dwMemInUse); 
            DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, szBuf);

			// Free the instance of the IHeap interface
			IHEAP_Release(pIHeap);
         }
         break;
		 
	  case USAGE_REALLOC:
	  case USAGE_STRDUP:
         {
			// 7. Usage: IHEAP_Realloc
            // 8. Usage: IHEAP_StrDup

			// IHEAP_Realloc increases the size of a previously allocated block of memory,
			// while IHEAP_StrDup makes a copy of a wide string (AECHAR) into a block of
			// memory it allocates.
			// As examples of how to use these functions, we first call IHEAP_StrDup to make
			// a copy of a string.  We then use IHEAP_Realloc to increase the size of the memory
			// block that holds the copy so that it can be used to hold a larger string.

			IHeap * pIHeap;
			AECHAR * pszStr, szWBuf[50];
			char szBuf[50];

			// Create an instance of the IHeap interface
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_HEAP, 
				  (void **)&pIHeap) != SUCCESS)
				return;

			// Create an example string in wide-string format
			STR_TO_WSTR("This is a text string", szWBuf, sizeof(szWBuf));

			// Use IHEAP_StrDup to make a copy of the string.
			// The function returns a pointer to a memory block
			// containing the copy.
 			if ((pszStr = IHEAP_StrDup(pIHeap, szWBuf)) == NULL)
				return;

			// Convert the copy to a standard string for our display function
            WSTR_TO_STR(pszStr, szBuf, WSTRLEN(pszStr) + 1);

			// Echo the string back to the screen
			DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, szBuf);

			// Create a bigger example string in wide-string format
			STR_TO_WSTR("This is a bigger text string", szWBuf, sizeof(szWBuf));

			// Increase the size of the memory block we used above for the
			// copy of the shorter string so that it will be large enough for
			// the bigger string
			if ((pszStr = IHEAP_Realloc(pIHeap, pszStr, (sizeof(AECHAR) * (WSTRLEN(szWBuf) + 1)))) == NULL)
				return;

			// Copy the bigger string into the newly expanded memory block
			WSTRCPY(pszStr, szWBuf);

			// Convert the copy to a standard string for our display function
            WSTR_TO_STR(pszStr, szBuf, WSTRLEN(pszStr) + 1);

			// Echo the string back to the screen
			DisplayOutput(pMe->a.m_pIShell, pMe->a.m_pIDisplay, 3, szBuf);

			// Free the memory used by the duplicated string
			IHEAP_Free(pIHeap, pszStr);

			// Free the instance of the IHeap interface
			IHEAP_Release(pIHeap);
         }
         break;

      default:
         return;
   }
            
   // Display heading at top of screen that indicates the example
   // selected, and display message at bottom instructing the user
   // to return to main menu by pressing UP or DOWN. 
   DisplayEvent (pMe, wParam);

   return;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function prints a heading at the top 
	of the screen to indicate which usage example was selected by the user, 
	It also prints a message on the bottom two lines of the screen instructing 
	the user to press the UP or DOWN key to return to
	the main menu to execute another example.
    
PROTOTYPE:
   static void DisplayEvent(CIHeapApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.
   wParam: [in]: Identifier of the selected example (example IDs are specified when the
                 main usage app menu is created).  If the value USAGE_ERASE_SCREEN is supplied
				 instead, this function clears the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void DisplayEvent (CIHeapApp *pMe, uint16 wParam)
{
   AEERect qrc;
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;
   AEERect rc; 

   // Make sure the pointers we'll be using are valid
   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
   rc.x = 5;
   rc.y = 5;
   rc.dx = di.cxScreen - 5;
   rc.dy = di.cyScreen - 5;

   // Specify the text string to be used for the example heading
   switch (wParam)
   {
      case USAGE_ERASE_SCREEN:
         IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&qrc);  //Clear the display
		 FREE(szBuf);
         return;
      case USAGE_BASIC_USAGE:
         STR_TO_WSTR("Basic Usage", szBuf, TEXT_BUFFER_SIZE);
         break; 
      case USAGE_CHECKAVAIL:
         STR_TO_WSTR("CheckAvail", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_FREE:
         STR_TO_WSTR("Free", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_GETMEMSTATS:
         STR_TO_WSTR("GetMemStats", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_MALLOC:
         STR_TO_WSTR("Malloc", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_MALLOCREC:
         STR_TO_WSTR("MallocRec", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_REALLOC:
         STR_TO_WSTR("Realloc", szBuf, TEXT_BUFFER_SIZE);
         break;
      case USAGE_STRDUP:
         STR_TO_WSTR("StrDup", szBuf, TEXT_BUFFER_SIZE);
         break;
      default:
         STR_TO_WSTR("Unsupported fn", szBuf, TEXT_BUFFER_SIZE);
   }

   // Print the heading at the top of the screen
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, 5, &rc,
	  IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

   // Print message at bottom of screen to instruct user to press UP
   // or DOWN to return to main menu
   STR_TO_WSTR("Press Up/Down", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 25, &rc,
		IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   STR_TO_WSTR("keys to continue..", szBuf, TEXT_BUFFER_SIZE);
   IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, qrc.dy - 15, &rc,
		IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
   IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);

   FREE(szBuf);
}
/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function constructs the menu control that allows the user to
	select a usage example to execute.  The command ID for each menu item
	is passed to the usage example function above when the user selects the
	menu item for that example.
    
PROTOTYPE:
   static void BuildMainMenu(CIHeapApp * pMe)

PARAMETERS:
   pMe:   [in]: Contains a pointer to the usage app data structure.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
static void BuildMainMenu(CIHeapApp *pMe)
{
	AEERect qrc;
	AEEDeviceInfo di;
    AECHAR * szBuf;

	// Make sure the pointers we'll be using are valid
    if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->m_pIMenu == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

	// Set Title
    STR_TO_WSTR("IHeap Examples", szBuf, TEXT_BUFFER_SIZE);
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);

   // Add Free to menu
   STR_TO_WSTR("2. Free", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_FREE, szBuf, 0);

   // Add Malloc to menu
   STR_TO_WSTR("3. Malloc", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_MALLOC, szBuf, 0);

   // Add MallocRec to menu
   STR_TO_WSTR("4. MallocRec", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_MALLOCREC, szBuf, 0);

   // Add CheckAvail to menu
   STR_TO_WSTR("5. CheckAvail", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_CHECKAVAIL, szBuf, 0);

   // Add GetMemStats to menu
   STR_TO_WSTR("6. GetMemStats", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_GETMEMSTATS, szBuf, 0);

   // Add Realloc to menu
   STR_TO_WSTR("7. Realloc", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_REALLOC, szBuf, 0);

   // Add StrDup to menu
   STR_TO_WSTR("8. StrDup", szBuf, TEXT_BUFFER_SIZE);
   IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_STRDUP, szBuf, 0);

   // Activate menu
   IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);

   FREE(szBuf);
}
/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr)

PARAMETERS:
   pIShell:   [in]: Contains a pointer to the IShell interface.
   pIDisplay: [in]: Contains a pointer to the IDisplay interface.
   nline:     [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR * szBuf;      // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   // Make sure the pointers we'll be using are valid
   if (pIShell == NULL || pIDisplay == NULL)
	   return;

   if ((szBuf = (AECHAR *) MALLOC(TEXT_BUFFER_SIZE * sizeof(AECHAR))) == NULL)
	   return;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, TEXT_BUFFER_SIZE);

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
	  {
		 FREE(szBuf);
         return;
      }

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   FREE(szBuf);
   return;   
} // End of DisplayOutput
