/*===========================================================================

FILE: inotifyusage.c

SERVICES: Sample applet using AEE

DESCRIPTION
  This file contains usage examples of INOTIFY interface. INOTIFY interface
  allows the user applet to register with the shell to be notified
  when a particular event has occured on the phone (i.e one such example
  of a notify service is TAPI). See ISHELL_RegisterNotify function descriptions 
  for more information on registering events with the shell.
  
  In this usage example, instead of registering to be notified by a handset 
  notification component (such as TAPI) we create a class which mimics 
  notifier functionality (that implements a notifier service). This notifier 
  class is built using basic building blocks of a typical notifier service.
  The notifier class implements the INotify interface instead of the IApplet
  interface. Hence, the notifier class will structurally differ from a 
  typical applet, which is event driven. It does not provide event handling 
  functionality and cannot be started like other user defined applets. But, 
  the Notifier class does provide the basic functionality to be loaded by 
  the applet manager at start time. The notifier functions and data strucutres
  are defined in notifier.c and ntifier.h files.

  High level Notification scenario:
    
    Consider a user applet A wants to be notified by the Shell when an event 
    has occured on notifiction service B.

    To achive this, Applet A would register with the Shell by invoking
    ISHELL_RegisterNotify() function with event corresponding to 
    NOTIFY_MASK_XXXX on notification service B. In turn the Shell sends 
    a message to the notify service B asking it to inform the Shell when 
    event corresponding to NOTIFY_MASK_XXXX has occured on service B.
    Service B then adds event to it's list of notify events. When an
    event on the service B has occured it looks up the notify event list 
    and inform the Shell that EVT_XXXX corresponding to NOTIFY_MASK_XXXX
    has occured by invoking ISHELL_Notify() function. The Shell then looks 
    up it's registered entry list and sends an EVT_NOTIFY event to the 
    appropriate registered applets. 

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"
#include "AEEAppGen.h"
#include "AEEMenu.h"
#include "AEEStdLib.h"
#include "AEEUsageAppIDs.h"
#include "notifier.h"           

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/
/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/
// Notify struct. This is the main struct for this applet. This will hold 
// all the data members that needs to be remembered throughout the life of
// the applet. 
// THE FIRST DATA MEMBER OF THIS STRUCT MUST BE AN AEEApplet OBJECT. 
// This is critical requirement which is related to an applet being
// dynamically loaded. The app developers should add their data members
// following the AEEApplet data member.
typedef struct _CINotifyApp
{
   AEEApplet	a;        // Mandatory AEEApplet data member appears first
   IMenuCtl *  m_pIMenu;
}CINotifyApp;

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/
// App Handle Event function
static boolean Notify_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam);

// App specific data alloc-init/free functions
static boolean INotify_InitAppData(IApplet* pMe);
static void INotify_FreeAppData(IApplet* pMe);

// The main Usage function where we register for the notify.
boolean NotifyUsage (CINotifyApp * pMe, uint16 wParam);

// Helper functions to build menu, and display events and strings.
void BuildMainMenu(CINotifyApp *pMe);
void DisplayEvent (CINotifyApp * pMe, uint16 wParam);
//void DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr);

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
                                                 // notifier class 
// App specific constants
#define USAGE_BASIC_USAGE         100
#define USAGE_ERASE_SCREEN        110   // Special case to erase screen

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to INotify_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData instance. 
   It is app developers responsibility to fill-in app data initialization code of 
   INotify_InitAppData(). App developer is also responsible to release memory allocated 
   for data contained in AppletData -- this can be done in INotify_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded
	pIShell: [in]: Contains pointer to the IShell interface. 
	pIModule: pin]: Contains pointer to the IModule interface to the current module 
      to which this app belongs
	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   // We want to load this App. So, invoke AEEApplet_New().To it, pass the
   // address of the app-specific handle event function.
   if(ClsId == AEECLSID_NOTIFY_APP){
      if(AEEApplet_New(sizeof(CINotifyApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)Notify_HandleEvent,(PFNFREEAPPDATA)INotify_FreeAppData)
         == TRUE)
      {
         INotify_InitAppData((IApplet*)*ppObj);
         return(AEE_SUCCESS);
      }
   }
   else if (ClsId == AEECLSID_NOTIFIER_APP)
   {
      // Invoke the notifier new. Notifier functions are defined on notifier.c file. 
      if (NotifierClass_New(sizeof(NotifierClass), ClsId, pIShell,po,(NotifierClass**)ppObj)
         == TRUE)
      {
         // Nothing to initialize
         return (AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION Notify_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean App_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean Notify_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CINotifyApp * pMe = (CINotifyApp*)pi;

   switch (eCode) 
	{
      case EVT_APP_START:
         // Create the IMenu interface object. When the applet is started the 
         // main menu is set up. When the user selects a menu item from the main 
         // menu, an EVT_COMMAND event is received by this applet. The processing for 
         // EVT_COMMAND event is found below.
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu)
            != SUCCESS)
         {
				return FALSE;
         }

         // Display the main menu. The main menu will include all the interface 
         // functions for the INotify interface.
			BuildMainMenu(pMe);
			return(TRUE);
      case EVT_APP_STOP:
         // Check if the IMenu interface is released. If not release it here.
         if (pMe->m_pIMenu != NULL)
         {
			   IMENUCTL_Release(pMe->m_pIMenu);
            pMe->m_pIMenu = NULL;
         }
         return(TRUE);
      case EVT_KEY:
         // We come here when a key is pressed on the handset keypad. Call 
         // the IMenuCtl handle event function to handle this key press.
         // If this key press corressponds to an Up/Down arrow 
         // key press, this event will be handled by the IMenuCtl object
         // in moving to the next menu item (in the given direction). 
         // If the key is neither Up/Down, the IMenuCtl the key press 
         // is not handled, hence, return FALSE.
			if(pMe->m_pIMenu != NULL && IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0))
				return TRUE;
         else
            return FALSE;
		case EVT_COMMAND:
         // When a menu item is selected this applet receives the
         // EVT_COMMAND event in the wParam field.
  		   switch(wParam)
         {
            case USAGE_BASIC_USAGE:
               // Call the usage function that exercises all the 
               // interface functions.
               NotifyUsage (pMe, wParam);   // Call NotifyUsage to process this event.
				   return TRUE;
			   default:
				   return FALSE;
         }
         break;
      case EVT_NOTIFY:
         {
            // We have received a notify event. Just display that we have received
            // a notify event. This event was registered with in the NotifyUsage
            // function by invoking ISHELL_RegisterNotify. 
            AEENotify * notifyData;   // dwParam = AEENotify *

            // "dwParam" contains the pointer to AEENotify struct. This needs to be
            // used to determine Event-specific data of the notifier. AEENotify 
            // definition is given below:
            // typedef struct
            // {
            //    AEECLSID    cls;           // Notifier class
            //    INotifier * pNotifier;     // Notifier object that issued the notify
            //    uint32      dwMask;        // Mask of bit(s) that occurred
            //    void *      pData;         // Event-specific data
            // } AEENotify;

            notifyData = (AEENotify *)dwParam; 

            // Check which notifier this event belongs to. 
            switch (notifyData->cls)
            {
               // It's possible that this applet is waiting on notify events
               // of several applets (in this example only one). 
               case AEECLSID_NOTIFIER_APP:
                  {
                     // Check what event occurred on the notifier. It's possible
                     // to have mutiple notify request outstanding from this applet
                     // to a given notify class (in this case there is only one).
                     if ((notifyData->dwMask & NOTIFY_MASK_EVENT) == NOTIFY_MASK_EVENT)
                     {
                        // The bits corresponding the event we requested are set.
                        // "Requester" below signifies the applet that requested the 
                        // notification by invoking ISHELL_RegisterNotify() function.
                        DisplayOutput (pMe->a.m_pIShell, pMe->a.m_pIDisplay, 5, "Requester: Received Notice!!!");
                     }
                  }
               default:
                  break;
            }
         }
      default:
         break;
   }
   return FALSE;
}



/*===========================================================================

FUNCTION InitAppData

DESCRIPTION
   This function initializes app specific dataallocates memory for app data (AppletData) and sets it to pAppData of AEEApplet.
   It also function . 
   App developer needs to fill-in the initialization code.

PROTOTYPE:
	boolean AllocAppData(IApplet* pMe);

PARAMETERS:
	pMe [in]: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pMe is not NULL

RETURN VALUE
  TRUE: If the app has app data is allocated and initialized successfully
  FALSE: Either app data could not be allocated or initialzied

SIDE EFFECTS
  none
===========================================================================*/
static boolean INotify_InitAppData(IApplet* pi)
{
   CINotifyApp * pMe = (CINotifyApp*)pi;

   // Initialize the MenuCtl pointer to NULL
   pMe->m_pIMenu = NULL;

   return TRUE;
}

/*===========================================================================

FUNCTION INotify_FreeAppData

DESCRIPTION
   Frees data contained in app data and memory for app data itself.
   App developer needs to free data contained in AppletData.

PROTOTYPE:
	boolean INotify_FreeAppData(IApplet* pMe);

PARAMETERS:
	pMe [in]: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

DEPENDENCIES
   Assumes pMe is not NULL

RETURN VALUE
  TRUE: If the app has app data initialized successfully
  FALSE: Either app data could not be initialzied

SIDE EFFECTS
  none
===========================================================================*/
static void INotify_FreeAppData(IApplet* pi)
{
   CINotifyApp * pMe = (CINotifyApp*)pi;

   if (pMe->m_pIMenu != NULL)
   {
      IMENUCTL_Release (pMe->m_pIMenu);
      pMe->m_pIMenu = NULL;
   }
}

/*===============================================================================
                     APP HELPER FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: NotifyUsage

DESCRIPTION
	This function encompasses all the usage examples of all the functions
   in code blocks switched using the BREW API function Id passed into this
   function. The code blocks will include almost all the variable declarations
   needed to demonstrate the usage of a given function. Within the code block
   all the needed interfaces are instantiated and released subsequent to their
   use.

   There are some exceptions to the above general rule, such as call-back functions,
   common helper function, or common display functions called from within the
   code block which are defined elsewhere in this file.

   There are some instances where an object instantiated within a code block is
   not released at the end of the code block, to be released at a later point
   (for example in a call-back function, in this case pointers to such objects
   are stored in the AEEApplet object to be released later).
   
PROTOTYPE:
   boolean NotifyUsage (CINotifyApp * pMe, uint16 wParam)

PARAMETERS:
	pMe: [in]: Contains a pointer to the CINotifyApp struct.

   wParam: [in]: Contains the Id of the function type to be used (contains the
      Id corresponding to the menu item selected on the emulator).

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the functions call was successful. FALSE otherwise

SIDE EFFECTS
  none
===========================================================================*/
boolean NotifyUsage (CINotifyApp * pMe, uint16 wParam)
{
   // Erase screen first for display output purposes.
   DisplayEvent (pMe, USAGE_ERASE_SCREEN);

   switch (wParam)
   {
      case USAGE_BASIC_USAGE:
         {
            // 1. Usage Basic Notifier Usage.
            AEECLSID clsNotifier= AEECLSID_NOTIFIER_APP;
            AEECLSID clsNotify = AEECLSID_NOTIFY_APP;
            // Set the notify event to be NOTIFY_MASK_EVENT. This event is used
            // as an example.
            uint32 dwMask = NOTIFY_MASK_EVENT;   

            // Register a notification 
            ISHELL_RegisterNotify (pMe->a.m_pIShell, clsNotify, clsNotifier, dwMask);
            // "Requester" below signifies the applet that requested the notification
            // by invoking ISHELL_RegisterNotify() function.
            DisplayOutput (pMe->a.m_pIShell, pMe->a.m_pIDisplay, 2, "Requester: Registered notify-event with Shell...");         
         }
         break;
      default:
         return FALSE;    
   }
            
   // Display above event. 
   DisplayEvent (pMe, wParam);

   return TRUE;
}

/*===========================================================================

FUNCTION: DisplayEvent

DESCRIPTION
    This function is a display function which sets up the initial screen
    for the display output for a given interface function. This displays
    the function name at the top of the screen and displays the
    strings "Press Up/Down" and "keys to continue.." at the bottom of 
    the screen. In the case of functions that have no output it also 
    displays the string "No Output..." in the middile of the screen.
    
PROTOTYPE:
   void DisplayEvent (CINotifyApp * pMe, uint16 wParam)

PARAMETERS:
   pMe:    [in]: Pointer to CINotifyApp interface object
   wParam: [in]: API function type Id selected by user

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void DisplayEvent (CINotifyApp * pMe, uint16 wParam)
{
   AEEDeviceInfo di;           // Device Info
   AECHAR szBuf[100];
   AEERect rc; 
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels

   if (pMe == NULL || pMe->a.m_pIShell == NULL || pMe->a.m_pIDisplay == NULL)
      return;

   ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
   rc.x = 0;
   rc.y = 0;
   rc.dx = di.cxScreen;
   rc.dy = di.cyScreen;
   
   if (pMe->a.m_pIDisplay)
   {
      // Get the font metrics info
      charHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
         &pnAscent, &pnDescent);

      switch (wParam)
      {
         case USAGE_ERASE_SCREEN:
            IDISPLAY_EraseRect(pMe->a.m_pIDisplay,&rc);  //Clear the display
            return;
         case USAGE_BASIC_USAGE:
            STR_TO_WSTR("Basic Usage", szBuf, sizeof(szBuf));
            break;
         default:
            STR_TO_WSTR("Unsupported fn", szBuf, sizeof(szBuf));
      }
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,5, 5, &rc,
		   IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);

      STR_TO_WSTR("Press END key", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, rc.dy - 
         2*(charHeight) , &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      STR_TO_WSTR("key to exit", szBuf, sizeof(szBuf));
		IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_NORMAL, szBuf, -1,10, rc.dy - 
         charHeight, &rc, IDF_ALIGN_NONE|  IDF_TEXT_TRANSPARENT);
      IDISPLAY_UpdateEx(pMe->a.m_pIDisplay,TRUE);

   }
}

/*===========================================================================

FUNCTION: BuildMainMenu

DESCRIPTION
    This function builds the main menu when this applet is started.
    It displays a simple menu with menu items corresponding to 
    each of the interface functions.
    
PROTOTYPE:
   void BuildMainMenu(CINotifyApp *pMe)

PARAMETERS:
   pMe: [in]: Pointer to CINotifyApp struct

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void BuildMainMenu(CINotifyApp *pMe)
{
   AEERect qrc;
	AEEDeviceInfo di;
   AECHAR szBuf[100];

   if (pMe->m_pIMenu == NULL || pMe->a.m_pIShell == NULL)
      return;

	// Set Title
   STR_TO_WSTR("INotify Examples", szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);

	//Set the Rectangle for the Menu
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&di);
	qrc.x = 0;
	qrc.y = 0;
	qrc.dx = di.cxScreen;
	qrc.dy = di.cyScreen;
	IMENUCTL_SetRect(pMe->m_pIMenu, &qrc);  

	//Add invidual entries to the Menu

   // Add Basic Usage to menu
   STR_TO_WSTR("1. Basic Usage", szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, USAGE_BASIC_USAGE, szBuf, 0);
			
   // Activate menu
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

/*===========================================================================

FUNCTION: DisplayOutput

DESCRIPTION
    This function displays an output string at a given line number on the
    screen. If the nline parameter is a negative value (-1) the string
    is displayed in the middle of the screen. If the "nline" value is larger
    than or equal to zero the "nline" value is multiplied by 15 and the 
    resulting value in pixels is set to the y-coordinate of the start of 
    the string display on the screen. If the string does not fit on one line
    the string wraps around to the next line (spaced rougly 10-15 pixels apart).
    By default 5 is used as the starting the x-coordinate of a displayed 
    string.

    How many characters that fit on one line is calculated for each line 
    that is wrapped around to the next line.

    Note: depending on the phone screen size and the fonts used for characters 
          the output might differ on different handsets (devices). Where some 
          handsets will have a smaller screen and large default fonts which will 
          cause partial overlapping of lines. This function does not try to address
          these issues (this is meant as a simple display function).
    
PROTOTYPE:
   void DisplayOutput(IShell *pIShell, IDisplay *pDisplay, int nline, char *pszStr)

PARAMETERS:
   pIShell:   [in]: Contains a pointer to the IShell interface.
   pIDisplay: [in]: Contains a pointer to the IDisplay interface.
   nline:     [in]: Contains the line number to start displaying the text. The line
        numbers are by default spaced 15 pixels apart along the y-axis.
   pszStr:    [in]: The character string to be displayed on the screen.

DEPENDENCIES
  None

RETURN VALUE
  None

SIDE EFFECTS
  None

===========================================================================*/
void DisplayOutput(IShell *pIShell, IDisplay *pIDisplay, int nline, char *pszStr)
{
   AEEDeviceInfo di; // Device Info
   AECHAR szBuf[200] = {0};      // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;

   // Get device information
   ISHELL_GetDeviceInfo(pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   STR_TO_WSTR ((char *)pszStr, szBuf, sizeof (szBuf));

   // If nlines is zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by charheight to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

      IDISPLAY_DrawText(pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line.
      IDISPLAY_Update(pIDisplay); //, TRUE);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }

      return;   
} // End of DisplayOutput
