/*===========================================================================

FILE: inotifier.c

SERVICES: Sample notifier class using AEE

DESCRIPTION
  This file contains a class that implements the INotifier interface.
  INotifier interface implements services that allows a user applet to 
  register with the shell to be notified when a particular event has occured 
  notifier service. See ISHELL_RegisterNotify function descriptions 
  for more information on registering events with the shell.
  
  In this usage example, instead of registering to be notified by a handset 
  notification component (such as TAPI) we create a class which mimics 
  notifier functionality (that implements a notifier service). This notifier 
  class is built using basic building blocks of a typical notifier service.
  The notifier class implements the INotify interface instead of the IApplet
  interface. Hence, the notifier class will structurally differ from a 
  typical applet, which is event driven. It does not provide event handling 
  functionality and cannot be started like other user defined applets. But, 
  the Notifier class does provide the basic functionality to be loaded by 
  the applet manager at start time.

  High level Notification scenario:
    
    Consider a user applet A wants to be notified by the Shell when an event 
    has occured on notifiction service B.

    To achive this, Applet A would register with the Shell by invoking
    ISHELL_RegisterNotify() function with event corresponding to 
    NOTIFY_MASK_XXXX on notification service B. In turn the Shell sends 
    a message to the notify service B asking it to inform the Shell when 
    event corresponding to NOTIFY_MASK_XXXX has occured on service B.
    Service B then adds event to it's list of notify events. When an
    event on the service B has occured it looks up the notify event list 
    and inform the Shell that EVT_XXXX corresponding to NOTIFY_MASK_XXXX
    has occured by invoking ISHELL_Notify() function. The Shell then looks 
    up it's registered entry list and sends an EVT_NOTIFY event to the 
    appropriate registered applets. 

PUBLIC CLASSES:  
   N/A


   	   Copyright  2000-2001 QUALCOMM Incorporated.
	                  All Rights Reserved.
                   QUALCOMM Proprietary/GTDR
===========================================================================*/

/*===============================================================================
                     INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */

#include "notifier.h"

/*===========================================================================

                      PUBLIC DATA DECLARATIONS

===========================================================================*/

/*-------------------------------------------------------------------
            Type Declarations
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
            Function Prototypes
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
            Global Constant Definitions
-------------------------------------------------------------------*/
 
/*-------------------------------------------------------------------
            Global Data Definitions
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
            Static variable Definitions
-------------------------------------------------------------------*/

/*===============================================================================
                     FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

Function:  NotifierClass_AddRef()

Description:
		This function increases the referecne count for the NotifierClass object

Prototype:
	uint32  NotifierClass_AddRef(Notifier* po)

Parameters:
   po: Pointer to Notifier interface whose reference count is to be incremented

Return Value:
	The updated reference count

Comments:  This function is a part of the Notifier class which implements
           a basic notification service.

Side Effects: None

==============================================================================*/
uint32  NotifierClass_AddRef(INotifier* po)
{
   return ++(((NotifierClass *)po)->m_nRefs);
}

/*===========================================================================

Function:  NotifierClass_Release()

Description:
		This function decreases the referecne count for the Notifier object. If
		the reference count reaches zero, it does the required cleanup

Prototype:
	uint32  NotifierClass_Release(Notifier* po)

Parameters:
   po: Pointer to the Notifier interface whose reference count needs to be
       decremented.

Return Value:
   The updated reference count

Comments:  This function is a part of the Notifier class which implements
           a basic notification service.

Side Effects: None

==============================================================================*/
uint32  NotifierClass_Release(INotifier* po)
{
	NotifierClass * pMe = (NotifierClass *)po;

	if (--pMe->m_nRefs != 0)
		return pMe->m_nRefs;

	//Release the interfaces created in NotifierClass_New()
   if (pMe->m_pIDisplay)
      IDISPLAY_Release(pMe->m_pIDisplay);   

   // Remove it from the list...
   AEEMod_ListRemove(pMe->m_pIModule, (AEEModObj *)pMe);

   IMODULE_Release(pMe->m_pIModule);

	// delete this NotifierClass object allocated in NotifierClass_New
   FREEOBJ(pMe);   
   
   ISHELL_Release(pMe->m_pIShell);   // Release the Shell

	return 0;
}

/*===========================================================================

Function:  NotifierClass_TimerCB()

Description:
		This function gets activated when the timer set in the 
      NotifierClass_SetMask() function expires. This timer expiration
      is used to mimic an event occuring on the Notifier-Service. And hence
      we inform the Shell by invoking the ISHELL_Notify() function that
      a previosly requested event has occursed.

Prototype:
	void NotifierClass_TimerCB (void *pi)

Parameters:
   pi: pointer to user data

Return Value: None

Comments:  This function is a part of the Notifier class which implements
           a basic notification service.

Side Effects: None

==============================================================================*/
void NotifierClass_TimerCB (void *pi)
{
   AEERect rect;       // Set up the rect coords for erasing/displaying
   NotifierClass * pMe = (NotifierClass*)pi;
   AEEDeviceInfo di;  // Device Info

   // Get device infor so that we can erase a section of the screen,
   ISHELL_GetDeviceInfo(pMe->m_pIShell,&di); 
   rect.x = 0;
   rect.y = di.cyScreen/8;
   rect.dx = di.cxScreen;
   rect.dy = di.cyScreen - (di.cyScreen/3);
   
   // First erase the screen where the text is to be printed
   IDISPLAY_EraseRect(pMe->m_pIDisplay,&rect);  //Clear the display

   // "Notifier" signifies the class that provides the event notification
   // services (i.e. TAPI)
   DisplayOutput (pMe->m_pIShell, pMe->m_pIDisplay, 2, "Notifier:Event notified to Shell...");

   // Notify the shell that the requested event has occured on the 
   // Notifier-object.
   ISHELL_Notify(pMe->m_pIShell, AEECLSID_NOTIFIER_APP, 
      pMe->m_dwNotifyMask, (void *) pMe);
}

/*===========================================================================

Function:  NotifierClass_SetMask()

Description:
   This function gets called by the Shell when an user applet has requested 
   a notification be done when a event of type "dwMask" has occured on the
   notification service.

   In a typical notification service, "dwMask" will be stored
   in an event list. When the corresponding event occurs a notification 
   is sent to the Shell informing of the event occuring on the notification
   service object.

   Instead in this object, a timer is set up to occur in 3 seconds. 
   This timer expiration triggers the NotifierClass_TimerCB()
   function to be called. In the the callback function the Shell is 
   notified that the requested event has occured on the notifier service 
   object. In turn the Shell will send the EVT_NOTIFY event to the user 
   applet which requested the event notify.

Prototype:
	void NotifierClass_SetMask(Notifier* po, const uint32 * dwMask)

Parameters:
   po     [in]: pointer to Notifier interface.
   dwMask [in]: contains a const pointer to the event mask

Return Value: None

Comments:  This function is a part of the Notifier class which implements
           a basic notification service.

Side Effects: None

==============================================================================*/
void NotifierClass_SetMask(INotifier* po, const uint32 * dwMask)
{
   NotifierClass * pMe = (NotifierClass *)po;
   int timerPeriod = 3000;             // Timer period in MS


   // just check if the notifier is the value sent by the user.
   if (*dwMask != NOTIFY_MASK_EVENT)
   {
      DisplayOutput (pMe->m_pIShell, pMe->m_pIDisplay, -1, "Notifier: Mask mismatch");
      return;
   }
   // Store the event that was requested for later use in the callback function.
   pMe->m_dwNotifyMask = *dwMask;

   // Start timer .
   ISHELL_SetTimer (pMe->m_pIShell, timerPeriod, NotifierClass_TimerCB, (void *)pMe);

   // "Notifier" siginifies the applet that provides the event notification
   // services (i.e. TAPI)
   DisplayOutput (pMe->m_pIShell, pMe->m_pIDisplay, 5, "Notifier: Waiting for event to occur (in 3 seconds)...");
}

/*===========================================================================

Function: NotifierClass_New

Description:
	This function is invoked when the notifier class is being loaded. It creates 
   a new instance of the NotifierClass structure. This function is invoked from 
   within the AEEClsCreateInstance() function.

Prototype:
	boolean NotifierClass_New(int16 nSize,
                             AEECLSID clsID,
							        IShell * pIShell, 
							        IModule * pIModule, 
							        NotifierClass **ppobj)

Parameters:
   nSize: [in]: Size of the notifier class object.
	clsID: [in]: Specifies the ClassID of this class
	pIShell: [in]: Contains a valid pointer to the AEE IShell interface
	pIModule: [in]: contains a vlaid pointer to the module to which this app belongs
	ppobj: [out]: On return, *ppobj points to a valid NotifierClass structure. This 
       allocation is done by this function

Dependancies: None

Return Value:
  TRUE: If successful
  FALSE: If failed

Comments:  This function is a part of the Notifier class which implements
           a basic notification service.

Side Effects: None

===========================================================================*/
boolean NotifierClass_New(int16 nSize,
                     AEECLSID clsID,
							IShell * pIShell, 
							IModule * pIModule, 
							NotifierClass **ppobj)
{
   NotifierClass * pMe = NULL;
   VTBL(INotifier) *appFuncs;
		
   *ppobj = NULL;

   //Validate parameters
   if (!pIShell || !pIModule)
      return FALSE;

   if(nSize < sizeof(NotifierClass))
      nSize += sizeof(NotifierClass);

   // Allocate memory for the notifier object and the INotifierVtbl
   if (NULL == (pMe = (NotifierClass*)MALLOC(nSize + sizeof(INotifierVtbl))))
      return FALSE;

   appFuncs = (INotifierVtbl *)((byte *)pMe + nSize);

	//Initialize the individual entries in the VTBL
	appFuncs->AddRef      = NotifierClass_AddRef; 
   appFuncs->Release     = NotifierClass_Release;
   appFuncs->SetMask     = NotifierClass_SetMask;

   INIT_VTBL(pMe, INotifier, *appFuncs);   // Initialize the VTBL

	//Initialize the data members
   pMe->m_nRefs      = 1;
   pMe->m_pIShell    = pIShell;
   pMe->m_pIModule   = pIModule;
   pMe->m_pIDisplay  = NULL;
	pMe->clsID = clsID;

   ISHELL_AddRef(pIShell);
   IMODULE_AddRef(pIModule);

   ISHELL_CreateInstance(pIShell, AEECLSID_DISPLAY, (void **) &pMe->m_pIDisplay);
  	if (!pMe->m_pIDisplay)
	{
		//Cleanup
		FREEOBJ(pMe);
      return FALSE;
	} 
	
   // Add it to the tracking list for the module...
   AEEMod_ListAdd(pIModule, (AEEModObj *)pMe);

   *ppobj = (NotifierClass*)pMe;  

   return TRUE;
}

